@@ -1,6 +1,6 @@
 /*
  * WPA Supplicant / Network configuration structures
- * Copyright (c) 2003-2005, Jouni Malinen <jkmaline@cc.hut.fi>
+ * Copyright (c) 2003-2006, Jouni Malinen <jkmaline@cc.hut.fi>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 2 as
@@ -253,10 +253,16 @@ struct wpa_ssid {
 	 * This file can have one or more trusted CA certificates. If ca_cert
 	 * and ca_path are not included, server certificate will not be
 	 * verified. This is insecure and a trusted CA certificate should
-	 * always be configured when using EAP-TLS/TTLS/PEAP.
+	 * always be configured when using EAP-TLS/TTLS/PEAP. Full path to the
+	 * file should be used since working directory may change when
+	 * wpa_supplicant is run in the background.
 	 *
 	 * Alternatively, a named configuration blob can be used by setting
 	 * this to blob://<blob name>.
+	 *
+	 * On Windows, trusted CA certificates can be loaded from the system
+	 * certificate store by setting this to cert_store://<name>, e.g.,
+	 * ca_cert="cert_store://CA" or ca_cert="cert_store://ROOT".
 	 */
 	u8 *ca_cert;
 
@@ -276,7 +282,9 @@ struct wpa_ssid {
 	 *
 	 * This field is used with EAP method that use TLS authentication.
 	 * Usually, this is only configured for EAP-TLS, even though this could
-	 * in theory be used with EAP-TTLS and EAP-PEAP, too.
+	 * in theory be used with EAP-TTLS and EAP-PEAP, too. Full path to the
+	 * file should be used since working directory may change when
+	 * wpa_supplicant is run in the background.
 	 *
 	 * Alternatively, a named configuration blob can be used by setting
 	 * this to blob://<blob name>.
@@ -288,7 +296,9 @@ struct wpa_ssid {
 	 *
 	 * When PKCS#12/PFX file (.p12/.pfx) is used, client_cert should be
 	 * commented out. Both the private key and certificate will be read
-	 * from the PKCS#12 file in this case.
+	 * from the PKCS#12 file in this case. Full path to the file should be
+	 * used since working directory may change when wpa_supplicant is run
+	 * in the background.
 	 *
 	 * Windows certificate store can be used by leaving client_cert out and
 	 * configuring private_key in one of the following formats:
@@ -320,7 +330,9 @@ struct wpa_ssid {
 	 * possible setup RSA to use ephemeral DH key exchange. In addition,
 	 * ciphers with DSA keys always use ephemeral DH keys. This can be used
 	 * to achieve forward secrecy. If the file is in DSA parameters format,
-	 * it will be automatically converted into DH params.
+	 * it will be automatically converted into DH params. Full path to the
+	 * file should be used since working directory may change when
+	 * wpa_supplicant is run in the background.
 	 *
 	 * Alternatively, a named configuration blob can be used by setting
 	 * this to blob://<blob name>.
@@ -361,7 +373,9 @@ struct wpa_ssid {
 	 * This file can have one or more trusted CA certificates. If ca_cert2
 	 * and ca_path2 are not included, server certificate will not be
 	 * verified. This is insecure and a trusted CA certificate should
-	 * always be configured.
+	 * always be configured. Full path to the file should be used since
+	 * working directory may change when wpa_supplicant is run in the
+	 * background.
 	 *
 	 * This field is like ca_cert, but used for phase 2 (inside
 	 * EAP-TTLS/PEAP/FAST tunnel) authentication.
@@ -389,7 +403,9 @@ struct wpa_ssid {
 	 * client_cert2 - File path to client certificate file
 	 *
 	 * This field is like client_cert, but used for phase 2 (inside
-	 * EAP-TTLS/PEAP/FAST tunnel) authentication.
+	 * EAP-TTLS/PEAP/FAST tunnel) authentication. Full path to the
+	 * file should be used since working directory may change when
+	 * wpa_supplicant is run in the background.
 	 *
 	 * Alternatively, a named configuration blob can be used by setting
 	 * this to blob://<blob name>.
@@ -400,7 +416,9 @@ struct wpa_ssid {
 	 * private_key2 - File path to client private key file
 	 *
 	 * This field is like private_key, but used for phase 2 (inside
-	 * EAP-TTLS/PEAP/FAST tunnel) authentication.
+	 * EAP-TTLS/PEAP/FAST tunnel) authentication. Full path to the
+	 * file should be used since working directory may change when
+	 * wpa_supplicant is run in the background.
 	 *
 	 * Alternatively, a named configuration blob can be used by setting
 	 * this to blob://<blob name>.
@@ -419,7 +437,9 @@ struct wpa_ssid {
 	 * dh_file2 - File path to DH/DSA parameters file (in PEM format)
 	 *
 	 * This field is like dh_file, but used for phase 2 (inside
-	 * EAP-TTLS/PEAP/FAST tunnel) authentication.
+	 * EAP-TTLS/PEAP/FAST tunnel) authentication. Full path to the
+	 * file should be used since working directory may change when
+	 * wpa_supplicant is run in the background.
 	 *
 	 * Alternatively, a named configuration blob can be used by setting
 	 * this to blob://<blob name>.
@@ -445,10 +465,10 @@ struct wpa_ssid {
 	/**
 	 * eap_methods - Allowed EAP methods
 	 *
-	 * Zero (EAP_TYPE_NONE) terminated list of allowed EAP methods or %NULL
-	 * if all methods are accepted.
+	 * (vendor=EAP_VENDOR_IETF,method=EAP_TYPE_NONE) terminated list of
+	 * allowed EAP methods or %NULL if all methods are accepted.
 	 */
-	u8 *eap_methods;
+	struct eap_method_type *eap_methods;
 
 	/**
 	 * phase1 - Phase 1 (outer authentication) parameters
@@ -687,7 +707,9 @@ struct wpa_ssid {
 	 * pac_file - File path or blob name for the PAC entries (EAP-FAST)
 	 *
 	 * wpa_supplicant will need to be able to create this file and write
-	 * updates to it when PAC is being provisioned or refreshed.
+	 * updates to it when PAC is being provisioned or refreshed. Full path
+	 * to the file should be used since working directory may change when
+	 * wpa_supplicant is run in the background.
 	 * Alternatively, a named configuration blob can be used by setting
 	 * this to blob://<blob name>.
 	 */
@@ -739,8 +761,20 @@ struct wpa_ssid {
 	 * ctrl_iface, e.g., with wpa_cli or wpa_gui).
 	 */
 	int disabled;
+
+	/**
+	 * stakey -  Whether STAKey negotiation for direct links is allowed
+	 *
+	 * This is only used when both RSN/WPA2 and IEEE 802.11e (QoS) are
+	 * enabled.
+	 *
+	 * 0 = disabled (default)
+	 * 1 = enabled
+	 */
+	int stakey;
 };
 
-int wpa_config_allowed_eap_method(struct wpa_ssid *ssid, int method);
+int wpa_config_allowed_eap_method(struct wpa_ssid *ssid, int vendor,
+				  u32 method);
 
 #endif /* CONFIG_SSID_H */
