@@ -13,13 +13,8 @@
  * See README and COPYING for more details.
  */
 
-#include <stdlib.h>
-#include <stdio.h>
-#include <unistd.h>
-#include <string.h>
+#include "includes.h"
 #include <sys/ioctl.h>
-#include <errno.h>
-#include <sys/socket.h>
 #if 0
 #include <netpacket/packet.h>
 #include <net/ethernet.h>     /* the L2 protocols */
@@ -27,7 +22,6 @@
 #include <linux/if_packet.h>
 #include <linux/if_ether.h>   /* The L2 protocols */
 #endif
-#include <netinet/in.h>
 #include <net/if.h>
 #include <typedefs.h>
 
