@@ -1,7 +1,7 @@
 /*
  * WPA Supplicant - driver interaction with Linux ndiswrapper
- * Copyright (c) 2004-2006, Giridhar Pemmasani <giri@lmc.cs.sunysb.edu>
- * Copyright (c) 2004-2006, Jouni Malinen <jkmaline@cc.hut.fi>
+ * Copyright (c) 2004-2005, Giridhar Pemmasani <giri@lmc.cs.sunysb.edu>
+ * Copyright (c) 2004-2005, Jouni Malinen <jkmaline@cc.hut.fi>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 2 as
@@ -13,12 +13,8 @@
  * See README and COPYING for more details.
  */
 
-#include <stdlib.h>
-#include <stdio.h>
-#include <unistd.h>
-#include <string.h>
+#include "includes.h"
 #include <sys/ioctl.h>
-#include <errno.h>
 #include <net/if_arp.h>
 
 #include "wireless_copy.h"
@@ -61,7 +57,6 @@ struct wpa_assoc_info
 	wpa_cipher group_suite;
 	wpa_key_mgmt key_mgmt_suite;
 	int auth_alg;
-	int mode;
 };
 
 #define PRIV_RESET	 		SIOCIWFIRSTPRIV+0
@@ -73,9 +68,6 @@ struct wpa_assoc_info
 #define WPA_SET_COUNTERMEASURES 	SIOCIWFIRSTPRIV+6
 #define WPA_DEAUTHENTICATE	 	SIOCIWFIRSTPRIV+7
 #define WPA_SET_AUTH_ALG	 	SIOCIWFIRSTPRIV+8
-#define WPA_INIT			SIOCIWFIRSTPRIV+9
-#define WPA_DEINIT			SIOCIWFIRSTPRIV+10
-#define WPA_GET_CAPA		 	SIOCIWFIRSTPRIV+11
 
 static int get_socket(void)
 {
@@ -240,7 +232,6 @@ wpa_ndiswrapper_associate(void *priv,
 	wpa_assoc_info.group_suite = params->group_suite;
 	wpa_assoc_info.key_mgmt_suite = params->key_mgmt_suite;
 	wpa_assoc_info.auth_alg = params->auth_alg;
-	wpa_assoc_info.mode = params->mode;
 
 	priv_req.u.data.pointer = (void *)&wpa_assoc_info;
 	priv_req.u.data.length = sizeof(wpa_assoc_info);
@@ -294,23 +285,6 @@ static int wpa_ndiswrapper_get_scan_results(void *priv,
 }
 
 
-static int wpa_ndiswrapper_get_capa(void *priv, struct wpa_driver_capa *capa)
-{
-	struct wpa_driver_ndiswrapper_data *drv = priv;
-	int ret = 0;
-	struct iwreq priv_req;
-
-	memset(&priv_req, 0, sizeof(priv_req));
-
-	priv_req.u.data.pointer = (void *) capa;
-	priv_req.u.data.length = sizeof(*capa);
-	if (iw_set_ext(drv, WPA_GET_CAPA, &priv_req) < 0)
-		ret = -1;
-	return ret;
-	
-}
-
-
 static void * wpa_ndiswrapper_init(void *ctx, const char *ifname)
 {
 	struct wpa_driver_ndiswrapper_data *drv;
@@ -365,5 +339,4 @@ const struct wpa_driver_ops wpa_driver_ndiswrapper_ops = {
 	.get_scan_results = wpa_ndiswrapper_get_scan_results,
 	.init = wpa_ndiswrapper_init,
 	.deinit = wpa_ndiswrapper_deinit,
-	.get_capa = wpa_ndiswrapper_get_capa,
 };
