@@ -18,12 +18,8 @@
  * ioctls, but require private ioctls for some of the functionality.
  */
 
-#include <stdlib.h>
-#include <stdio.h>
-#include <unistd.h>
-#include <string.h>
+#include "includes.h"
 #include <sys/ioctl.h>
-#include <errno.h>
 #include <net/if_arp.h>
 
 #include "wireless_copy.h"
@@ -51,10 +47,6 @@ struct wpa_driver_wext_data {
 	struct wpa_driver_capa capa;
 	int has_capability;
 	int we_version_compiled;
-
-	/* for set_auth_alg fallback */
-	int use_crypt;
-	int auth_alg_fallback;
 };
 
 
@@ -77,7 +69,7 @@ static int wpa_driver_wext_set_auth_param(struct wpa_driver_wext_data *drv,
 		perror("ioctl[SIOCSIWAUTH]");
 		fprintf(stderr, "WEXT auth param %d value 0x%x - ",
 			idx, value);
-		ret = errno == EOPNOTSUPP ? -2 : -1;
+		ret = -1;
 	}
 
 	return ret;
@@ -163,11 +155,8 @@ int wpa_driver_wext_get_ssid(void *priv, u8 *ssid)
 		if (ret > 32)
 			ret = 32;
 		/* Some drivers include nul termination in the SSID, so let's
-		 * remove it here before further processing. WE-21 changes this
-		 * to explicitly require the length _not_ to include nul
-		 * termination. */
-		if (ret > 0 && ssid[ret - 1] == '\0' &&
-		    drv->we_version_compiled < 21)
+		 * remove it here before further processing. */
+		if (ret > 0 && ssid[ret - 1] == '\0')
 			ret--;
 	}
 
@@ -199,19 +188,12 @@ int wpa_driver_wext_set_ssid(void *priv, const u8 *ssid, size_t ssid_len)
 	memset(buf, 0, sizeof(buf));
 	memcpy(buf, ssid, ssid_len);
 	iwr.u.essid.pointer = (caddr_t) buf;
-	if (drv->we_version_compiled < 21) {
-		/* For historic reasons, set SSID length to include one extra
-		 * character, C string nul termination, even though SSID is
-		 * really an octet string that should not be presented as a C
-		 * string. Some Linux drivers decrement the length by one and
-		 * can thus end up missing the last octet of the SSID if the
-		 * length is not incremented here. WE-21 changes this to
-		 * explicitly require the length _not_ to include nul
-		 * termination. */
-		if (ssid_len)
-			ssid_len++;
-	}
-	iwr.u.essid.length = ssid_len;
+	/* For historic reasons, set SSID length to include one extra
+	 * character, C string nul termination, even though SSID is really an
+	 * octet string that should not be presented as a C string. Some Linux
+	 * drivers decrement the length by one and can thus end up missing the
+	 * last octet of the SSID if the length is not incremented here. */
+	iwr.u.essid.length = ssid_len ? ssid_len + 1 : 0;
 
 	if (ioctl(drv->ioctl_sock, SIOCSIWESSID, &iwr) < 0) {
 		perror("ioctl[SIOCSIWESSID]");
@@ -307,6 +289,15 @@ wpa_driver_wext_event_wireless_custom(struct wpa_driver_wext_data *drv,
 	done:
 		free(data.assoc_info.resp_ies);
 		free(data.assoc_info.req_ies);
+#ifdef CONFIG_STAKEY
+	} else if (strncmp(custom, "STAKey-Request=", 15) == 0) {
+		if (hwaddr_aton(custom + 15, data.stakey_request.peer)) {
+			wpa_printf(MSG_DEBUG, "WEXT: unrecognized "
+				   "STAKey-Request '%s'", custom + 15);
+			return;
+		}
+		wpa_supplicant_event(ctx, EVENT_STAKEY_REQUEST, &data);
+#endif /* CONFIG_STAKEY */
 	}
 }
 
@@ -932,11 +923,7 @@ static int wpa_scan_result_compar(const void *a, const void *b)
 
 	/* use freq for channel preference */
 
-	/* all things being equal, use signal level; if signal levels are
-	 * identical, use quality values since some drivers may only report
-	 * that value and leave the signal level zero */
-	if (wb->level == wa->level)
-		return wb->qual - wb->qual;
+	/* all things being equal, use signal level */
 	return wb->level - wa->level;
 }
 
@@ -1468,7 +1455,6 @@ static int wpa_driver_wext_set_drop_unencrypted(void *priv,
 {
 	struct wpa_driver_wext_data *drv = priv;
 	wpa_printf(MSG_DEBUG, "%s", __FUNCTION__);
-	drv->use_crypt = enabled;
 	return wpa_driver_wext_set_auth_param(drv, IW_AUTH_DROP_UNENCRYPTED,
 					      enabled);
 }
@@ -1572,50 +1558,6 @@ static int wpa_driver_wext_keymgmt2wext(int keymgmt)
 }
 
 
-static int
-wpa_driver_wext_auth_alg_fallback(struct wpa_driver_wext_data *drv,
-				  struct wpa_driver_associate_params *params)
-{
-	struct iwreq iwr;
-	int ret = 0;
-
-	wpa_printf(MSG_DEBUG, "WEXT: Driver did not support "
-		   "SIOCSIWAUTH for AUTH_ALG, trying SIOCSIWENCODE");
-
-	memset(&iwr, 0, sizeof(iwr));
-	strncpy(iwr.ifr_name, drv->ifname, IFNAMSIZ);
-	/* Just changing mode, not actual keys */
-	iwr.u.encoding.flags = 0;
-	iwr.u.encoding.pointer = (caddr_t) NULL;
-	iwr.u.encoding.length = 0;
-
-	/*
-	 * Note: IW_ENCODE_{OPEN,RESTRICTED} can be interpreted to mean two
-	 * different things. Here they are used to indicate Open System vs.
-	 * Shared Key authentication algorithm. However, some drivers may use
-	 * them to select between open/restricted WEP encrypted (open = allow
-	 * both unencrypted and encrypted frames; restricted = only allow
-	 * encrypted frames).
-	 */
-
-	if (!drv->use_crypt) {
-		iwr.u.encoding.flags |= IW_ENCODE_DISABLED;
-	} else {
-		if (params->auth_alg & AUTH_ALG_OPEN_SYSTEM)
-			iwr.u.encoding.flags |= IW_ENCODE_OPEN;
-		if (params->auth_alg & AUTH_ALG_SHARED_KEY)
-			iwr.u.encoding.flags |= IW_ENCODE_RESTRICTED;
-	}
-
-	if (ioctl(drv->ioctl_sock, SIOCSIWENCODE, &iwr) < 0) {
-		perror("ioctl[SIOCSIWENCODE]");
-		ret = -1;
-	}
-
-	return ret;
-}
-
-
 static int
 wpa_driver_wext_associate(void *priv,
 			  struct wpa_driver_associate_params *params)
@@ -1627,14 +1569,6 @@ wpa_driver_wext_associate(void *priv,
 
 	wpa_printf(MSG_DEBUG, "%s", __FUNCTION__);
 
-	/*
-	 * If the driver did not support SIOCSIWAUTH, fallback to
-	 * SIOCSIWENCODE here.
-	 */
-	if (drv->auth_alg_fallback &&
-	    wpa_driver_wext_auth_alg_fallback(drv, params) < 0)
-		ret = -1;
-
 	if (!params->bssid &&
 	    wpa_driver_wext_set_bssid(drv, NULL) < 0)
 		ret = -1;
@@ -1668,10 +1602,7 @@ wpa_driver_wext_associate(void *priv,
 	if (wpa_driver_wext_set_auth_param(drv,
 					   IW_AUTH_KEY_MGMT, value) < 0)
 		ret = -1;
-	value = params->key_mgmt_suite != KEY_MGMT_NONE ||
-		params->pairwise_suite != CIPHER_NONE ||
-		params->group_suite != CIPHER_NONE ||
-		params->wpa_ie_len;
+	value = params->key_mgmt_suite != KEY_MGMT_NONE;
 	if (wpa_driver_wext_set_auth_param(drv,
 					   IW_AUTH_PRIVACY_INVOKED, value) < 0)
 		ret = -1;
@@ -1704,7 +1635,7 @@ wpa_driver_wext_associate(void *priv,
 static int wpa_driver_wext_set_auth_alg(void *priv, int auth_alg)
 {
 	struct wpa_driver_wext_data *drv = priv;
-	int algs = 0, res;
+	int algs = 0;
 
 	if (auth_alg & AUTH_ALG_OPEN_SYSTEM)
 		algs |= IW_AUTH_ALG_OPEN_SYSTEM;
@@ -1717,10 +1648,8 @@ static int wpa_driver_wext_set_auth_alg(void *priv, int auth_alg)
 		algs = IW_AUTH_ALG_OPEN_SYSTEM;
 	}
 
-	res = wpa_driver_wext_set_auth_param(drv, IW_AUTH_80211_AUTH_ALG,
-					     algs);
-	drv->auth_alg_fallback = res == -2;
-	return res;
+	return wpa_driver_wext_set_auth_param(drv,
+					      IW_AUTH_80211_AUTH_ALG, algs);
 }
 
 
