@@ -1,6 +1,6 @@
 /*
  * WPA Supplicant / EAP state machines (RFC 4137)
- * Copyright (c) 2004-2005, Jouni Malinen <jkmaline@cc.hut.fi>
+ * Copyright (c) 2004-2006, Jouni Malinen <jkmaline@cc.hut.fi>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 2 as
@@ -10,21 +10,9 @@
  * license.
  *
  * See README and COPYING for more details.
- *
- * This file implements the Peer State Machine as defined in RFC 4137. The used
- * states and state transitions match mostly with the RFC. However, there are
- * couple of additional transitions for working around small issues noticed
- * during testing. These exceptions are explained in comments within the
- * functions in this file. The method functions, m.func(), are similar to the
- * ones used in RFC 4137, but some small changes have used here to optimize
- * operations and to add functionality needed for fast re-authentication
- * (session resumption).
  */
 
-#include <stdlib.h>
-#include <stdio.h>
-#include <string.h>
-#include <ctype.h>
+#include "includes.h"
 
 #include "common.h"
 #include "eap_i.h"
@@ -34,7 +22,10 @@
 #include "crypto.h"
 #include "pcsc_funcs.h"
 #include "wpa_ctrl.h"
+#include "state_machine.h"
 
+#define STATE_MACHINE_DATA struct eap_sm
+#define STATE_MACHINE_DEBUG_PREFIX "EAP"
 
 #define EAP_MAX_AUTH_ROUNDS 50
 
@@ -78,6 +69,9 @@ extern const struct eap_method eap_method_fast;
 #ifdef EAP_PAX
 extern const struct eap_method eap_method_pax;
 #endif
+#ifdef EAP_VENDOR_TEST
+extern const struct eap_method eap_method_vendor_test;
+#endif
 
 static const struct eap_method *eap_methods[] =
 {
@@ -120,59 +114,44 @@ static const struct eap_method *eap_methods[] =
 #ifdef EAP_PAX
 	&eap_method_pax,
 #endif
+#ifdef EAP_VENDOR_TEST
+	&eap_method_vendor_test,
+#endif
+	NULL
 };
-#define NUM_EAP_METHODS (sizeof(eap_methods) / sizeof(eap_methods[0]))
+#define NUM_EAP_METHODS ((sizeof(eap_methods) / sizeof(eap_methods[0])) - 1)
 
 
 /**
  * eap_sm_get_eap_methods - Get EAP method based on type number
+ * @vendor: EAP Vendor-Id (0 = IETF)
  * @method: EAP type number
  * Returns: Pointer to EAP method of %NULL if not found
  */
-const struct eap_method * eap_sm_get_eap_methods(int method)
+const struct eap_method * eap_sm_get_eap_methods(int vendor, EapType method)
 {
-	int i;
+	size_t i;
 	for (i = 0; i < NUM_EAP_METHODS; i++) {
-		if (eap_methods[i]->method == method)
+		if (eap_methods[i]->vendor == vendor &&
+		    eap_methods[i]->method == method)
 			return eap_methods[i];
 	}
 	return NULL;
 }
 
 
-static Boolean eap_sm_allowMethod(struct eap_sm *sm, EapType method);
+static Boolean eap_sm_allowMethod(struct eap_sm *sm, int vendor,
+				  EapType method);
 static u8 * eap_sm_buildNak(struct eap_sm *sm, int id, size_t *len);
 static void eap_sm_processIdentity(struct eap_sm *sm, const u8 *req,
 				   size_t len);
 static void eap_sm_processNotify(struct eap_sm *sm, const u8 *req, size_t len);
 static u8 * eap_sm_buildNotify(struct eap_sm *sm, int id, size_t *len);
 static void eap_sm_parseEapReq(struct eap_sm *sm, const u8 *req, size_t len);
-static const char * eap_sm_method_state_txt(EapMethodState state);
-static const char * eap_sm_decision_txt(EapDecision decision);
+static const char * eap_sm_method_state_txt(int state);
+static const char * eap_sm_decision_txt(int decision);
 
 
-/* Definitions for clarifying state machine implementation */
-#define SM_STATE(machine, state) \
-static void sm_ ## machine ## _ ## state ## _Enter(struct eap_sm *sm, \
-	int global)
-
-#define SM_ENTRY(machine, state) \
-if (!global || sm->machine ## _state != machine ## _ ## state) { \
-	sm->changed = TRUE; \
-	wpa_printf(MSG_DEBUG, "EAP: " #machine " entering state " #state); \
-} \
-sm->machine ## _state = machine ## _ ## state;
-
-#define SM_ENTER(machine, state) \
-sm_ ## machine ## _ ## state ## _Enter(sm, 0)
-#define SM_ENTER_GLOBAL(machine, state) \
-sm_ ## machine ## _ ## state ## _Enter(sm, 1)
-
-#define SM_STEP(machine) \
-static void sm_ ## machine ## _Step(struct eap_sm *sm)
-
-#define SM_STEP_RUN(machine) sm_ ## machine ## _Step(sm)
-
 
 static Boolean eapol_get_bool(struct eap_sm *sm, enum eapol_bool_var var)
 {
@@ -219,11 +198,6 @@ static void eap_deinit_prev_method(struct eap_sm *sm, const char *txt)
 }
 
 
-/*
- * This state initializes state machine variables when the machine is
- * activated (portEnabled = TRUE). This is also used when re-starting
- * authentication (eapRestart == TRUE).
- */
 SM_STATE(EAP, INITIALIZE)
 {
 	SM_ENTRY(EAP, INITIALIZE);
@@ -262,11 +236,6 @@ SM_STATE(EAP, INITIALIZE)
 }
 
 
-/*
- * This state is reached whenever service from the lower layer is interrupted
- * or unavailable (portEnabled == FALSE). Immediate transition to INITIALIZE
- * occurs when the port becomes enabled.
- */
 SM_STATE(EAP, DISABLED)
 {
 	SM_ENTRY(EAP, DISABLED);
@@ -274,21 +243,12 @@ SM_STATE(EAP, DISABLED)
 }
 
 
-/*
- * The state machine spends most of its time here, waiting for something to
- * happen. This state is entered unconditionally from INITIALIZE, DISCARD, and
- * SEND_RESPONSE states.
- */
 SM_STATE(EAP, IDLE)
 {
 	SM_ENTRY(EAP, IDLE);
 }
 
 
-/*
- * This state is entered when an EAP packet is received (eapReq == TRUE) to
- * parse the packet header.
- */
 SM_STATE(EAP, RECEIVED)
 {
 	const u8 *eapReqData;
@@ -302,88 +262,90 @@ SM_STATE(EAP, RECEIVED)
 }
 
 
-/*
- * This state is entered when a request for a new type comes in. Either the
- * correct method is started, or a Nak response is built.
- */
 SM_STATE(EAP, GET_METHOD)
 {
+	int reinit, method;
+
 	SM_ENTRY(EAP, GET_METHOD);
-	if (eap_sm_allowMethod(sm, sm->reqMethod)) {
-		int reinit = 0;
-		/*
-		 * RFC 4137 does not define specific operation for fast
-		 * re-authentication (session resumption). The design here is
-		 * to allow the previously used method data to be maintained
-		 * for re-authentication if the method support session
-		 * resumption. Otherwise, the previously used method data is
-		 * freed and a new method is allocated here.
-		 */
-		if (sm->fast_reauth &&
-		    sm->m && sm->m->method == sm->reqMethod &&
-		    sm->m->has_reauth_data &&
-		    sm->m->has_reauth_data(sm, sm->eap_method_priv)) {
-			wpa_printf(MSG_DEBUG, "EAP: Using previous method data"
-				   " for fast re-authentication");
-			reinit = 1;
-		} else
-			eap_deinit_prev_method(sm, "GET_METHOD");
-		sm->selectedMethod = sm->reqMethod;
-		if (sm->m == NULL)
-			sm->m = eap_sm_get_eap_methods(sm->selectedMethod);
-		if (sm->m) {
-			wpa_printf(MSG_DEBUG, "EAP: Initialize selected EAP "
-				   "method (%d, %s)",
-				   sm->selectedMethod, sm->m->name);
-			if (reinit)
-				sm->eap_method_priv = sm->m->init_for_reauth(
-					sm, sm->eap_method_priv);
-			else
-				sm->eap_method_priv = sm->m->init(sm);
-			if (sm->eap_method_priv == NULL) {
-				struct wpa_ssid *config = eap_get_config(sm);
-				wpa_msg(sm->msg_ctx, MSG_INFO,
-					"EAP: Failed to initialize EAP method "
-					"%d (%s)",
-					sm->selectedMethod, sm->m->name);
-				sm->m = NULL;
-				sm->methodState = METHOD_NONE;
-				sm->selectedMethod = EAP_TYPE_NONE;
-				if (sm->reqMethod == EAP_TYPE_TLS &&
-				    config &&
-				    (config->pending_req_pin ||
-				     config->pending_req_passphrase)) {
-					/*
-					 * Return without generating Nak in
-					 * order to allow entering of PIN code
-					 * or passphrase to retry the current
-					 * EAP packet.
-					 */
-					wpa_printf(MSG_DEBUG, "EAP: Pending "
-						   "PIN/passphrase request - "
-						   "skip Nak");
-					return;
-				}
-			} else {
-				sm->methodState = METHOD_INIT;
-				wpa_msg(sm->msg_ctx, MSG_INFO,
-					WPA_EVENT_EAP_METHOD
-					"EAP method %d (%s) selected",
-					sm->selectedMethod, sm->m->name);
-				return;
-			}
-		}
+
+	if (sm->reqMethod == EAP_TYPE_EXPANDED)
+		method = sm->reqVendorMethod;
+	else
+		method = sm->reqMethod;
+
+	if (!eap_sm_allowMethod(sm, sm->reqVendor, method)) {
+		wpa_printf(MSG_DEBUG, "EAP: vendor %u method %u not allowed",
+			   sm->reqVendor, method);
+		goto nak;
 	}
 
+	if (sm->fast_reauth &&
+	    sm->m && sm->m->vendor == sm->reqVendor &&
+	    sm->m->method == method &&
+	    sm->m->has_reauth_data &&
+	    sm->m->has_reauth_data(sm, sm->eap_method_priv)) {
+		wpa_printf(MSG_DEBUG, "EAP: using previous method data"
+			   " for fast re-authentication");
+		reinit = 1;
+	} else {
+		eap_deinit_prev_method(sm, "GET_METHOD");
+		reinit = 0;
+	}
+
+	sm->selectedMethod = sm->reqMethod;
+	if (sm->m == NULL)
+		sm->m = eap_sm_get_eap_methods(sm->reqVendor, method);
+	if (!sm->m) {
+		wpa_printf(MSG_DEBUG, "EAP: Could not find selected method: "
+			   "vendor %d method %d",
+			   sm->reqVendor, method);
+		goto nak;
+	}
+
+	wpa_printf(MSG_DEBUG, "EAP: initialize selected EAP method: "
+		   "vendor %u method %u (%s)",
+		   sm->reqVendor, method, sm->m->name);
+	if (reinit)
+		sm->eap_method_priv = sm->m->init_for_reauth(
+			sm, sm->eap_method_priv);
+	else
+		sm->eap_method_priv = sm->m->init(sm);
+
+	if (sm->eap_method_priv == NULL) {
+		struct wpa_ssid *config = eap_get_config(sm);
+		wpa_msg(sm->msg_ctx, MSG_INFO,
+			"EAP: Failed to initialize EAP method: vendor %u "
+			"method %u (%s)",
+			sm->reqVendor, method, sm->m->name);
+		sm->m = NULL;
+		sm->methodState = METHOD_NONE;
+		sm->selectedMethod = EAP_TYPE_NONE;
+		if (sm->reqMethod == EAP_TYPE_TLS && config &&
+		    (config->pending_req_pin ||
+		     config->pending_req_passphrase)) {
+			/*
+			 * Return without generating Nak in order to allow
+			 * entering of PIN code or passphrase to retry the
+			 * current EAP packet.
+			 */
+			wpa_printf(MSG_DEBUG, "EAP: Pending PIN/passphrase "
+				   "request - skip Nak");
+			return;
+		}
+	} else {
+		sm->methodState = METHOD_INIT;
+		wpa_msg(sm->msg_ctx, MSG_INFO, WPA_EVENT_EAP_METHOD
+			"EAP vendor %u method %u (%s) selected",
+			sm->reqVendor, method, sm->m->name);
+		return;
+	}
+
+nak:
 	free(sm->eapRespData);
 	sm->eapRespData = eap_sm_buildNak(sm, sm->reqId, &sm->eapRespDataLen);
 }
 
 
-/*
- * The method processing happens here. The request from the authenticator is
- * processed, and an appropriate response packet is built.
- */
 SM_STATE(EAP, METHOD)
 {
 	u8 *eapReqData;
@@ -398,20 +360,8 @@ SM_STATE(EAP, METHOD)
 
 	eapReqData = eapol_get_eapReqData(sm, &eapReqDataLen);
 
-	/*
-	 * Get ignore, methodState, decision, allowNotifications, and
-	 * eapRespData. RFC 4137 uses three separate method procedure (check,
-	 * process, and buildResp) in this state. These have been combined into
-	 * a single function call to m->process() in order to optimize EAP
-	 * method implementation interface a bit. These procedures are only
-	 * used from within this METHOD state, so there is no need to keep
-	 * these as separate C functions.
-	 *
-	 * The RFC 4137 procedures return values as follows:
-	 * ignore = m.check(eapReqData)
-	 * (methodState, decision, allowNotifications) = m.process(eapReqData)
-	 * eapRespData = m.buildResp(reqId)
-	 */
+	/* Get ignore, methodState, decision, allowNotifications, and
+	 * eapRespData. */
 	memset(&ret, 0, sizeof(ret));
 	ret.ignore = sm->ignore;
 	ret.methodState = sm->methodState;
@@ -443,10 +393,6 @@ SM_STATE(EAP, METHOD)
 }
 
 
-/*
- * This state signals the lower layer that a response packet is ready to be
- * sent.
- */
 SM_STATE(EAP, SEND_RESPONSE)
 {
 	SM_ENTRY(EAP, SEND_RESPONSE);
@@ -469,10 +415,6 @@ SM_STATE(EAP, SEND_RESPONSE)
 }
 
 
-/*
- * This state signals the lower layer that the request was discarded, and no
- * response packet will be sent at this time.
- */
 SM_STATE(EAP, DISCARD)
 {
 	SM_ENTRY(EAP, DISCARD);
@@ -481,9 +423,6 @@ SM_STATE(EAP, DISCARD)
 }
 
 
-/*
- * Handles requests for Identity method and builds a response.
- */
 SM_STATE(EAP, IDENTITY)
 {
 	const u8 *eapReqData;
@@ -498,9 +437,6 @@ SM_STATE(EAP, IDENTITY)
 }
 
 
-/*
- * Handles requests for Notification method and builds a response.
- */
 SM_STATE(EAP, NOTIFICATION)
 {
 	const u8 *eapReqData;
@@ -515,9 +451,6 @@ SM_STATE(EAP, NOTIFICATION)
 }
 
 
-/*
- * This state retransmits the previous response packet.
- */
 SM_STATE(EAP, RETRANSMIT)
 {
 	SM_ENTRY(EAP, RETRANSMIT);
@@ -534,11 +467,6 @@ SM_STATE(EAP, RETRANSMIT)
 }
 
 
-/*
- * This state is entered in case of a successful completion of authentication
- * and state machine waits here until port is disabled or EAP authentication is
- * restarted.
- */
 SM_STATE(EAP, SUCCESS)
 {
 	SM_ENTRY(EAP, SUCCESS);
@@ -566,10 +494,6 @@ SM_STATE(EAP, SUCCESS)
 }
 
 
-/*
- * This state is entered in case of a failure and state machine waits here
- * until port is disabled or EAP authentication is restarted.
- */
 SM_STATE(EAP, FAILURE)
 {
 	SM_ENTRY(EAP, FAILURE);
@@ -620,9 +544,6 @@ static int eap_success_workaround(struct eap_sm *sm, int reqId, int lastId)
 }
 
 
-/*
- * RFC 4137 - Appendix A.1: EAP Peer State Machine - State transitions
- */
 SM_STEP(EAP)
 {
 	int duplicate;
@@ -633,14 +554,6 @@ SM_STEP(EAP)
 	else if (!eapol_get_bool(sm, EAPOL_portEnabled) || sm->force_disabled)
 		SM_ENTER_GLOBAL(EAP, DISABLED);
 	else if (sm->num_rounds > EAP_MAX_AUTH_ROUNDS) {
-		/* RFC 4137 does not place any limit on number of EAP messages
-		 * in an authentication session. However, some error cases have
-		 * ended up in a state were EAP messages were sent between the
-		 * peer and server in a loop (e.g., TLS ACK frame in both
-		 * direction). Since this is quite undesired outcome, limit the
-		 * total number of EAP round-trips and abort authentication if
-		 * this limit is exceeded.
-		 */
 		if (sm->num_rounds == EAP_MAX_AUTH_ROUNDS + 1) {
 			wpa_msg(sm->msg_ctx, MSG_INFO, "EAP: more than %d "
 				"authentication rounds - abort",
@@ -658,11 +571,6 @@ SM_STEP(EAP)
 			SM_ENTER(EAP, INITIALIZE);
 		break;
 	case EAP_IDLE:
-		/*
-		 * The first three transitions are from RFC 4137. The last two
-		 * are local additions to handle special cases with LEAP and
-		 * PEAP server not sending EAP-Success in some cases.
-		 */
 		if (eapol_get_bool(sm, EAPOL_eapReq))
 			SM_ENTER(EAP, RECEIVED);
 		else if ((eapol_get_bool(sm, EAPOL_altAccept) &&
@@ -706,15 +614,10 @@ SM_STEP(EAP)
 			duplicate = 0;
 		}
 
-		/*
-		 * Two special cases below for LEAP are local additions to work
-		 * around odd LEAP behavior (EAP-Success in the middle of
-		 * authentication and then swapped roles). Other transitions
-		 * are based on RFC 4137.
-		 */
-		if (sm->rxSuccess && sm->decision != DECISION_FAIL &&
+		if (sm->rxSuccess &&
 		    (sm->reqId == sm->lastId ||
-		     eap_success_workaround(sm, sm->reqId, sm->lastId)))
+		     eap_success_workaround(sm, sm->reqId, sm->lastId)) &&
+		    sm->decision != DECISION_FAIL)
 			SM_ENTER(EAP, SUCCESS);
 		else if (sm->methodState != METHOD_CONT &&
 			 ((sm->rxFailure &&
@@ -785,27 +688,35 @@ SM_STEP(EAP)
 }
 
 
-static Boolean eap_sm_allowMethod(struct eap_sm *sm, EapType method)
+static Boolean eap_sm_allowMethod(struct eap_sm *sm, int vendor,
+				  EapType method)
 {
 	struct wpa_ssid *config = eap_get_config(sm);
-	int i;
+	size_t i;
 
-	if (!wpa_config_allowed_eap_method(config, method))
+	if (!wpa_config_allowed_eap_method(config, vendor, method)) {
+		wpa_printf(MSG_DEBUG, "EAP: configuration does not allow: "
+			   "vendor %u method %u", vendor, method);
 		return FALSE;
+	}
 	for (i = 0; i < NUM_EAP_METHODS; i++) {
-		if (eap_methods[i]->method == method)
+		if (eap_methods[i]->vendor == vendor &&
+		    eap_methods[i]->method == method)
 			return TRUE;
 	}
+	wpa_printf(MSG_DEBUG, "EAP: not included in build: "
+		   "vendor %u method %u", vendor, method);
 	return FALSE;
 }
 
 
-static u8 * eap_sm_buildNak(struct eap_sm *sm, int id, size_t *len)
+static u8 *eap_sm_buildNak(struct eap_sm *sm, int id, size_t *len)
 {
 	struct wpa_ssid *config = eap_get_config(sm);
 	struct eap_hdr *resp;
 	u8 *pos;
-	int i, found = 0;
+	int found = 0;
+	size_t i;
 
 	wpa_printf(MSG_DEBUG, "EAP: Building EAP-Nak (requested type %d not "
 		   "allowed)", sm->reqMethod);
@@ -821,7 +732,7 @@ static u8 * eap_sm_buildNak(struct eap_sm *sm, int id, size_t *len)
 
 	for (i = 0; i < NUM_EAP_METHODS; i++) {
 		if (eap_methods[i]->method != sm->reqMethod &&
-		    wpa_config_allowed_eap_method(config,
+		    wpa_config_allowed_eap_method(config, EAP_VENDOR_IETF,
 						  eap_methods[i]->method)) {
 			*pos++ = eap_methods[i]->method;
 			(*len)++;
@@ -851,13 +762,6 @@ static void eap_sm_processIdentity(struct eap_sm *sm, const u8 *req,
 	wpa_msg(sm->msg_ctx, MSG_INFO, WPA_EVENT_EAP_STARTED
 		"EAP authentication started");
 
-	/*
-	 * RFC 3748 - 5.1: Identity
-	 * Data field may contain a displayable message in UTF-8. If this
-	 * includes NUL-character, only the data before that should be
-	 * displayed. Some EAP implementasitons may piggy-back additional
-	 * options after the NUL.
-	 */
 	/* TODO: could save displayable message so that it can be shown to the
 	 * user in case of interaction is required */
 	wpa_hexdump_ascii(MSG_DEBUG, "EAP: EAP-Request Identity data",
@@ -870,7 +774,8 @@ static int eap_sm_imsi_identity(struct eap_sm *sm, struct wpa_ssid *ssid)
 	int aka = 0;
 	char imsi[100];
 	size_t imsi_len;
-	u8 *pos = ssid->eap_methods;
+	struct eap_method_type *m = ssid->eap_methods;
+	int i;
 
 	imsi_len = sizeof(imsi);
 	if (scard_get_imsi(sm->scard_ctx, imsi, &imsi_len)) {
@@ -880,12 +785,13 @@ static int eap_sm_imsi_identity(struct eap_sm *sm, struct wpa_ssid *ssid)
 
 	wpa_hexdump_ascii(MSG_DEBUG, "IMSI", (u8 *) imsi, imsi_len);
 
-	while (pos && *pos != EAP_TYPE_NONE) {
-		if (*pos == EAP_TYPE_AKA) {
+	for (i = 0; m && (m[i].vendor != EAP_VENDOR_IETF ||
+			  m[i].method != EAP_TYPE_NONE); i++) {
+		if (m[i].vendor == EAP_VENDOR_IETF &&
+		    m[i].method == EAP_TYPE_AKA) {
 			aka = 1;
 			break;
 		}
-		pos++;
 	}
 
 	free(ssid->identity);
@@ -926,8 +832,8 @@ static int eap_sm_get_scard_identity(struct eap_sm *sm, struct wpa_ssid *ssid)
  * eap_sm_buildIdentity - Build EAP-Identity/Response for the current network
  * @sm: Pointer to EAP state machine allocated with eap_sm_init()
  * @id: EAP identifier for the packet
- * @len: Pointer to a variable that will be set to the length of the response
- * @encrypted: Whether the packet is for encrypted tunnel (EAP phase 2)
+ * @len: Pointer to variable that will be set to the length of the response
+ * @encrypted: Whether the packet is for enrypted tunnel (EAP phase 2)
  * Returns: Pointer to the allocated EAP-Identity/Response packet or %NULL on
  * failure
  *
@@ -982,6 +888,7 @@ u8 * eap_sm_buildIdentity(struct eap_sm *sm, int id, size_t *len,
 		}
 	}
 
+
 	*len = sizeof(struct eap_hdr) + 1 + identity_len;
 	resp = malloc(*len);
 	if (resp == NULL)
@@ -1003,8 +910,7 @@ static void eap_sm_processNotify(struct eap_sm *sm, const u8 *req, size_t len)
 	const struct eap_hdr *hdr = (const struct eap_hdr *) req;
 	const u8 *pos;
 	char *msg;
-	size_t msg_len;
-	int i;
+	size_t i, msg_len;
 
 	pos = (const u8 *) (hdr + 1);
 	pos++;
@@ -1028,7 +934,7 @@ static void eap_sm_processNotify(struct eap_sm *sm, const u8 *req, size_t len)
 }
 
 
-static u8 * eap_sm_buildNotify(struct eap_sm *sm, int id, size_t *len)
+static u8 *eap_sm_buildNotify(struct eap_sm *sm, int id, size_t *len)
 {
 	struct eap_hdr *resp;
 	u8 *pos;
@@ -1053,10 +959,13 @@ static void eap_sm_parseEapReq(struct eap_sm *sm, const u8 *req, size_t len)
 {
 	const struct eap_hdr *hdr;
 	size_t plen;
+	u8 *pos;
 
-	sm->rxReq = sm->rxResp = sm->rxSuccess = sm->rxFailure = FALSE;
+	sm->rxReq = sm->rxSuccess = sm->rxFailure = FALSE;
 	sm->reqId = 0;
 	sm->reqMethod = EAP_TYPE_NONE;
+	sm->reqVendor = EAP_VENDOR_IETF;
+	sm->reqVendorMethod = EAP_TYPE_NONE;
 
 	if (req == NULL || len < sizeof(*hdr))
 		return;
@@ -1073,24 +982,42 @@ static void eap_sm_parseEapReq(struct eap_sm *sm, const u8 *req, size_t len)
 	sm->reqId = hdr->identifier;
 
 	if (sm->workaround) {
-		md5_vector(1, (const u8 **) &req, &len, sm->req_md5);
+		md5_vector(1, (const u8 **) &req, &plen, sm->req_md5);
 	}
 
 	switch (hdr->code) {
 	case EAP_CODE_REQUEST:
+		if (plen < sizeof(*hdr) + 1) {
+			wpa_printf(MSG_DEBUG, "EAP: Too short EAP-Request - "
+				   "no Type field");
+			return;
+		}
 		sm->rxReq = TRUE;
-		if (plen > sizeof(*hdr))
-			sm->reqMethod = *((u8 *) (hdr + 1));
-		wpa_printf(MSG_DEBUG, "EAP: Received EAP-Request method=%d "
-			   "id=%d", sm->reqMethod, sm->reqId);
+		pos = (u8 *) (hdr + 1);
+		sm->reqMethod = *pos++;
+		if (sm->reqMethod == EAP_TYPE_EXPANDED) {
+			if (plen < sizeof(*hdr) + 8) {
+				wpa_printf(MSG_DEBUG, "EAP: Ignored truncated "
+					   "expanded EAP-Packet (plen=%lu)",
+					   (unsigned long) plen);
+				return;
+			}
+			sm->reqVendor = WPA_GET_BE24(pos);
+			pos += 3;
+			sm->reqVendorMethod = WPA_GET_BE32(pos);
+		}
+		wpa_printf(MSG_DEBUG, "EAP: Received EAP-Request id=%d "
+			   "method=%u vendor=%u vendorMethod=%u",
+			   sm->reqId, sm->reqMethod, sm->reqVendor,
+			   sm->reqVendorMethod);
 		break;
 	case EAP_CODE_RESPONSE:
 		if (sm->selectedMethod == EAP_TYPE_LEAP) {
-			/*
-			 * LEAP differs from RFC 4137 by using reversed roles
-			 * for mutual authentication and because of this, we
-			 * need to accept EAP-Response frames if LEAP is used.
-			 */
+			if (plen < sizeof(*hdr) + 1) {
+				wpa_printf(MSG_DEBUG, "EAP: Too short "
+					   "EAP-Response - no Type field");
+				return;
+			}
 			sm->rxResp = TRUE;
 			if (plen > sizeof(*hdr))
 				sm->reqMethod = *((u8 *) (hdr + 1));
@@ -1126,10 +1053,7 @@ static void eap_sm_parseEapReq(struct eap_sm *sm, const u8 *req, size_t len)
  * Returns: Pointer to the allocated EAP state machine or %NULL on failure
  *
  * This function allocates and initializes an EAP state machine. In addition,
- * this initializes TLS library for the new EAP state machine. eapol_cb pointer
- * will be in use until eap_sm_deinit() is used to deinitialize this EAP state
- * machine. Consequently, the caller must make sure that this data structure
- * remains alive while the EAP state machine is active.
+ * this initializes TLS library for the new EAP state machine.
  */
 struct eap_sm * eap_sm_init(void *eapol_ctx, struct eapol_callbacks *eapol_cb,
 			    void *msg_ctx, struct eap_config *conf)
@@ -1174,7 +1098,9 @@ void eap_sm_deinit(struct eap_sm *sm)
 	if (sm == NULL)
 		return;
 	eap_deinit_prev_method(sm, "EAP deinit");
-	eap_sm_abort(sm);
+	free(sm->lastRespData);
+	free(sm->eapRespData);
+	free(sm->eapKeyData);
 	tls_deinit(sm->ssl_ctx);
 	free(sm);
 }
@@ -1211,8 +1137,6 @@ int eap_sm_step(struct eap_sm *sm)
  */
 void eap_sm_abort(struct eap_sm *sm)
 {
-	free(sm->lastRespData);
-	sm->lastRespData = NULL;
 	free(sm->eapRespData);
 	sm->eapRespData = NULL;
 	free(sm->eapKeyData);
@@ -1255,7 +1179,7 @@ static const char * eap_sm_state_txt(int state)
 }
 
 
-static const char * eap_sm_method_state_txt(EapMethodState state)
+static const char * eap_sm_method_state_txt(int state)
 {
 	switch (state) {
 	case METHOD_NONE:
@@ -1274,7 +1198,7 @@ static const char * eap_sm_method_state_txt(EapMethodState state)
 }
 
 
-static const char * eap_sm_decision_txt(EapDecision decision)
+static const char * eap_sm_decision_txt(int decision)
 {
 	switch (decision) {
 	case DECISION_FAIL:
@@ -1292,10 +1216,10 @@ static const char * eap_sm_decision_txt(EapDecision decision)
 /**
  * eap_sm_get_status - Get EAP state machine status
  * @sm: Pointer to EAP state machine allocated with eap_sm_init()
- * @buf: Buffer for status information
- * @buflen: Maximum buffer length
- * @verbose: Whether to include verbose status information
- * Returns: Number of bytes written to buf.
+ * @buf: buffer for status information
+ * @buflen: maximum buffer length
+ * @verbose: whether to include verbose status information
+ * Returns: number of bytes written to buf.
  *
  * Query EAP state machine for status information. This function fills in a
  * text area with current status information from the EAPOL state machine. If
@@ -1319,7 +1243,8 @@ int eap_sm_get_status(struct eap_sm *sm, char *buf, size_t buflen, int verbose)
 			name = sm->m->name;
 		} else {
 			const struct eap_method *m =
-				eap_sm_get_eap_methods(sm->selectedMethod);
+				eap_sm_get_eap_methods(EAP_VENDOR_IETF,
+						       sm->selectedMethod);
 			if (m)
 				name = m->name;
 			else
@@ -1570,35 +1495,41 @@ void eap_sm_notify_ctrl_attached(struct eap_sm *sm)
 /**
  * eap_get_type - Get EAP type for the given EAP method name
  * @name: EAP method name, e.g., TLS
+ * @vendor: Buffer for returning EAP Vendor-Id
  * Returns: EAP method type or %EAP_TYPE_NONE if not found
  *
  * This function maps EAP type names into EAP type numbers based on the list of
  * EAP methods included in the build.
  */
-u8 eap_get_type(const char *name)
+EapType eap_get_type(const char *name, int *vendor)
 {
-	int i;
+	size_t i;
 	for (i = 0; i < NUM_EAP_METHODS; i++) {
-		if (strcmp(eap_methods[i]->name, name) == 0)
+		if (strcmp(eap_methods[i]->name, name) == 0) {
+			*vendor = eap_methods[i]->vendor;
 			return eap_methods[i]->method;
+		}
 	}
+	*vendor = EAP_VENDOR_IETF;
 	return EAP_TYPE_NONE;
 }
 
 
 /**
  * eap_get_name - Get EAP method name for the given EAP type
+ * @vendor: EAP Vendor-Id (0 = IETF)
  * @type: EAP method type
  * Returns: EAP method name, e.g., TLS, or %NULL if not found
  *
  * This function maps EAP type numbers into EAP type names based on the list of
  * EAP methods included in the build.
  */
-const char * eap_get_name(EapType type)
+const char * eap_get_name(int vendor, EapType type)
 {
-	int i;
+	size_t i;
 	for (i = 0; i < NUM_EAP_METHODS; i++) {
-		if (eap_methods[i]->method == type)
+		if (eap_methods[i]->vendor == vendor &&
+		    eap_methods[i]->method == type)
 			return eap_methods[i]->name;
 	}
 	return NULL;
@@ -1615,7 +1546,7 @@ const char * eap_get_name(EapType type)
 size_t eap_get_names(char *buf, size_t buflen)
 {
 	char *pos, *end;
-	int i;
+	size_t i;
 
 	pos = buf;
 	end = pos + buflen;
@@ -1629,8 +1560,10 @@ size_t eap_get_names(char *buf, size_t buflen)
 }
 
 
-static int eap_allowed_phase2_type(int type)
+static int eap_allowed_phase2_type(int vendor, int type)
 {
+	if (vendor != EAP_VENDOR_IETF)
+		return 0;
 	return type != EAP_TYPE_PEAP && type != EAP_TYPE_TTLS &&
 		type != EAP_TYPE_FAST;
 }
@@ -1639,17 +1572,22 @@ static int eap_allowed_phase2_type(int type)
 /**
  * eap_get_phase2_type - Get EAP type for the given EAP phase 2 method name
  * @name: EAP method name, e.g., MD5
+ * @vendor: Buffer for returning EAP Vendor-Id
  * Returns: EAP method type or %EAP_TYPE_NONE if not found
  *
  * This function maps EAP type names into EAP type numbers that are allowed for
  * Phase 2, i.e., for tunneled authentication. Phase 2 is used, e.g., with
  * EAP-PEAP, EAP-TTLS, and EAP-FAST.
  */
-u8 eap_get_phase2_type(const char *name)
+u32 eap_get_phase2_type(const char *name, int *vendor)
 {
-	u8 type = eap_get_type(name);
-	if (eap_allowed_phase2_type(type))
+	int v;
+	u8 type = eap_get_type(name, &v);
+	if (eap_allowed_phase2_type(v, type)) {
+		*vendor = v;
 		return type;
+	}
+	*vendor = EAP_VENDOR_IETF;
 	return EAP_TYPE_NONE;
 }
 
@@ -1657,29 +1595,35 @@ u8 eap_get_phase2_type(const char *name)
 /**
  * eap_get_phase2_types - Get list of allowed EAP phase 2 types
  * @config: Pointer to a network configuration
- * @count: Pointer to a variable to be filled with number of returned EAP types
+ * @count: Pointer to variable filled with number of returned EAP types
  * Returns: Pointer to allocated type list or %NULL on failure
  *
  * This function generates an array of allowed EAP phase 2 (tunneled) types for
  * the given network configuration.
  */
-u8 *eap_get_phase2_types(struct wpa_ssid *config, size_t *count)
+struct eap_method_type * eap_get_phase2_types(struct wpa_ssid *config,
+					      size_t *count)
 {
-	u8 *buf, method;
-	int i;
+	struct eap_method_type *buf;
+	u32 method;
+	int vendor;
+	size_t i;
 
 	*count = 0;
-	buf = malloc(NUM_EAP_METHODS);
+	buf = malloc(NUM_EAP_METHODS * sizeof(struct eap_method_type));
 	if (buf == NULL)
 		return NULL;
 
 	for (i = 0; i < NUM_EAP_METHODS; i++) {
+		vendor = eap_methods[i]->vendor;
 		method = eap_methods[i]->method;
-		if (eap_allowed_phase2_type(method)) {
-			if (method == EAP_TYPE_TLS && config &&
+		if (eap_allowed_phase2_type(vendor, method)) {
+			if (vendor == EAP_VENDOR_IETF &&
+			    method == EAP_TYPE_TLS && config &&
 			    config->private_key2 == NULL)
 				continue;
-			buf[*count] = method;
+			buf[*count].vendor = vendor;
+			buf[*count].method = method;
 			(*count)++;
 		}
 	}
@@ -1691,7 +1635,7 @@ u8 *eap_get_phase2_types(struct wpa_ssid *config, size_t *count)
 /**
  * eap_set_fast_reauth - Update fast_reauth setting
  * @sm: Pointer to EAP state machine allocated with eap_sm_init()
- * @enabled: 1 = Fast reauthentication is enabled, 0 = Disabled
+ * @enabled: 1 = fast reauthentication is enabled, 0 = disabled
  */
 void eap_set_fast_reauth(struct eap_sm *sm, int enabled)
 {
@@ -1748,8 +1692,6 @@ void eap_notify_success(struct eap_sm *sm)
 		sm->EAP_state = EAP_SUCCESS;
 	}
 }
-
-
 /**
  * eap_notify_lower_layer_success - Notification of lower layer success
  * @sm: Pointer to EAP state machine allocated with eap_sm_init()
@@ -1832,7 +1774,7 @@ u8 * eap_get_eapRespData(struct eap_sm *sm, size_t *len)
 /**
  * eap_sm_register_scard_ctx - Notification of smart card context
  * @sm: Pointer to EAP state machine allocated with eap_sm_init()
- * @ctx: Context data for smart card operations
+ * @ctx: context data for smart card operations
  *
  * Notify EAP state machines of context data for smart card operations. This
  * context data will be used as a parameter for scard_*() functions.
@@ -1846,36 +1788,62 @@ void eap_register_scard_ctx(struct eap_sm *sm, void *ctx)
 
 /**
  * eap_hdr_validate - Validate EAP header
+ * @vendor: Expected EAP Vendor-Id (0 = IETF)
  * @eap_type: Expected EAP type number
  * @msg: EAP frame (starting with EAP header)
  * @msglen: Length of msg
- * @plen: Pointer to variable to contain the returned payload length
+ * @plen: Pointer for return payload length
  * Returns: Pointer to EAP payload (after type field), or %NULL on failure
  *
  * This is a helper function for EAP method implementations. This is usually
- * called in the beginning of struct eap_method::process() function to verify
- * that the received EAP request packet has a valid header.
+ * called in the beginning of struct eap_method::process() function.
  */
-const u8 * eap_hdr_validate(EapType eap_type, const u8 *msg, size_t msglen,
-			    size_t *plen)
+const u8 * eap_hdr_validate(int vendor, EapType eap_type,
+			    const u8 *msg, size_t msglen, size_t *plen)
 {
 	const struct eap_hdr *hdr;
 	const u8 *pos;
 	size_t len;
 
 	hdr = (const struct eap_hdr *) msg;
-	pos = (const u8 *) (hdr + 1);
-	if (msglen < sizeof(*hdr) + 1 || *pos != eap_type) {
-		wpa_printf(MSG_INFO, "EAP: Invalid frame type");
-		return NULL;
-	}
+
 	len = be_to_host16(hdr->length);
 	if (len < sizeof(*hdr) + 1 || len > msglen) {
 		wpa_printf(MSG_INFO, "EAP: Invalid EAP length");
 		return NULL;
 	}
-	*plen = len - sizeof(*hdr) - 1;
-	return pos + 1;
+
+	pos = (const u8 *) (hdr + 1);
+
+	if (*pos == EAP_TYPE_EXPANDED) {
+		int exp_vendor;
+		u32 exp_type;
+		if (len < sizeof(*hdr) + 8) {
+			wpa_printf(MSG_INFO, "EAP: Invalid expanded EAP "
+				   "length");
+			return NULL;
+		}
+		pos++;
+		exp_vendor = WPA_GET_BE24(pos);
+		pos += 3;
+		exp_type = WPA_GET_BE32(pos);
+		pos += 4;
+		if (exp_vendor != vendor || exp_type != (u32) eap_type) {
+			wpa_printf(MSG_INFO, "EAP: Invalid expanded frame "
+				   "type");
+			return NULL;
+		}
+
+		*plen = len - sizeof(*hdr) + 8;
+		return pos;
+	} else {
+		if (vendor != EAP_VENDOR_IETF || *pos != eap_type) {
+			wpa_printf(MSG_INFO, "EAP: Invalid frame type");
+			return NULL;
+		}
+		*plen = len - sizeof(*hdr) - 1;
+		return pos + 1;
+	}
 }
 
 
