@@ -12,9 +12,7 @@
  * See README and COPYING for more details.
  */
 
-#include <stdlib.h>
-#include <stdio.h>
-#include <string.h>
+#include "includes.h"
 
 #include "common.h"
 #include "eap_i.h"
@@ -105,8 +103,8 @@ struct eap_fast_data {
 	void *phase2_priv;
 	int phase2_success;
 
-	u8 phase2_type;
-	u8 *phase2_types;
+	struct eap_method_type phase2_type;
+	struct eap_method_type *phase2_types;
 	size_t num_phase2_types;
 	int resuming; /* starting a resumed session */
 	struct eap_fast_key_block_auth *key_block_a;
@@ -472,8 +470,7 @@ static void eap_fast_write(char **buf, char **pos, size_t *buf_len,
 			   const char *field, const u8 *data,
 			   size_t len, int txt)
 {
-	int i;
-	size_t need;
+	size_t i, need;
 
 	if (data == NULL || *buf == NULL)
 		return;
@@ -615,7 +612,8 @@ static void * eap_fast_init(struct eap_sm *sm)
 
 	if (config && config->phase2) {
 		char *start, *pos, *buf;
-		u8 method, *methods = NULL, *_methods;
+		struct eap_method_type *methods = NULL, *_methods;
+		u8 method;
 		size_t num_methods = 0;
 		start = buf = strdup(config->phase2);
 		if (buf == NULL) {
@@ -623,6 +621,7 @@ static void * eap_fast_init(struct eap_sm *sm)
 			return NULL;
 		}
 		while (start && *start != '\0') {
+			int vendor;
 			pos = strstr(start, "auth=");
 			if (pos == NULL)
 				break;
@@ -635,21 +634,24 @@ static void * eap_fast_init(struct eap_sm *sm)
 			pos = strchr(start, ' ');
 			if (pos)
 				*pos++ = '\0';
-			method = eap_get_phase2_type(start);
-			if (method == EAP_TYPE_NONE) {
+			method = eap_get_phase2_type(start, &vendor);
+			if (vendor == EAP_VENDOR_IETF &&
+			    method == EAP_TYPE_NONE) {
 				wpa_printf(MSG_ERROR, "EAP-FAST: Unsupported "
 					   "Phase2 method '%s'", start);
 			} else {
 				num_methods++;
-				_methods = realloc(methods, num_methods);
+				_methods = realloc(
+					methods,
+					num_methods * sizeof(*methods));
 				if (_methods == NULL) {
 					free(methods);
-					free(buf);
 					eap_fast_deinit(sm, data);
 					return NULL;
 				}
 				methods = _methods;
-				methods[num_methods - 1] = method;
+				methods[num_methods - 1].vendor = vendor;
+				methods[num_methods - 1].method = method;
 			}
 
 			start = pos;
@@ -668,8 +670,10 @@ static void * eap_fast_init(struct eap_sm *sm)
 		return NULL;
 	}
 	wpa_hexdump(MSG_DEBUG, "EAP-FAST: Phase2 EAP types",
-		    data->phase2_types, data->num_phase2_types);
-	data->phase2_type = EAP_TYPE_NONE;
+		    (u8 *) data->phase2_types,
+		    data->num_phase2_types * sizeof(struct eap_method_type));
+	data->phase2_type.vendor = EAP_VENDOR_IETF;
+	data->phase2_type.method = EAP_TYPE_NONE;
 
 	if (eap_tls_ssl_init(sm, &data->ssl, config)) {
 		wpa_printf(MSG_INFO, "EAP-FAST: Failed to initialize SSL.");
@@ -771,7 +775,8 @@ static int eap_fast_phase2_nak(struct eap_sm *sm,
 
 	wpa_printf(MSG_DEBUG, "EAP-FAST: Phase 2 Request: Nak type=%d", *pos);
 	wpa_hexdump(MSG_DEBUG, "EAP-FAST: Allowed Phase2 EAP types",
-		    data->phase2_types, data->num_phase2_types);
+		    (u8 *) data->phase2_types,
+		    data->num_phase2_types * sizeof(struct eap_method_type));
 	*resp_len = sizeof(struct eap_hdr) + 1 + data->num_phase2_types;
 	*resp = malloc(*resp_len);
 	if (*resp == NULL)
@@ -824,7 +829,8 @@ static int eap_fast_set_tls_master_secret(struct eap_sm *sm,
 	size_t seed_len, server_random_len;
 
 	if (data->tls_master_secret_set || !data->current_pac ||
-	    tls_connection_get_keys(sm->ssl_ctx, data->ssl.conn, &keys)) {
+	    tls_connection_get_keys(sm->ssl_ctx, data->ssl.conn, &keys) ||
+	    keys.client_random == NULL) {
 		return 0;
 	}
 
@@ -858,7 +864,6 @@ static int eap_fast_set_tls_master_secret(struct eap_sm *sm,
 	wpa_hexdump(MSG_DEBUG, "EAP-FAST: server_random",
 		    server_random, server_random_len);
 
-
 	seed_len = keys.client_random_len + server_random_len;
 	seed = malloc(seed_len);
 	if (seed == NULL)
@@ -891,22 +896,28 @@ static u8 * eap_fast_derive_key(struct eap_sm *sm, struct eap_ssl_data *data,
 				char *label, size_t len)
 {
 	struct tls_keys keys;
-	u8 *rnd;
-	u8 *out;
+	u8 *rnd = NULL, *out;
 	int block_size;
 
-	if (tls_connection_get_keys(sm->ssl_ctx, data->conn, &keys))
-		return NULL;
 	block_size = tls_connection_get_keyblock_size(sm->ssl_ctx, data->conn);
 	if (block_size < 0)
 		return NULL;
+
 	out = malloc(block_size + len);
-	rnd = malloc(keys.client_random_len + keys.server_random_len);
-	if (out == NULL || rnd == NULL) {
-		free(out);
-		free(rnd);
+	if (out == NULL)
 		return NULL;
-	}
+
+	if (tls_connection_prf(sm->ssl_ctx, data->conn, label, 1, out, len) ==
+	    0)
+		return out;
+
+	if (tls_connection_get_keys(sm->ssl_ctx, data->conn, &keys))
+		goto fail;
+
+	rnd = malloc(keys.client_random_len + keys.server_random_len);
+	if (rnd == NULL)
+		goto fail;
+
 	memcpy(rnd, keys.server_random, keys.server_random_len);
 	memcpy(rnd + keys.server_random_len, keys.client_random,
 	       keys.client_random_len);
@@ -915,14 +926,16 @@ static u8 * eap_fast_derive_key(struct eap_sm *sm, struct eap_ssl_data *data,
 			"expansion", keys.master_key, keys.master_key_len);
 	if (tls_prf(keys.master_key, keys.master_key_len,
 		    label, rnd, keys.client_random_len +
-		    keys.server_random_len, out, block_size + len)) {
-		free(rnd);
-		free(out);
-		return NULL;
-	}
+		    keys.server_random_len, out, block_size + len))
+		goto fail;
 	free(rnd);
 	memmove(out, out + block_size, len);
 	return out;
+
+fail:
+	free(rnd);
+	free(out);
+	return NULL;
 }
 
 
@@ -1000,27 +1013,37 @@ static int eap_fast_phase2_request(struct eap_sm *sm,
 		*resp = eap_sm_buildIdentity(sm, req->identifier, resp_len, 1);
 		break;
 	default:
-		if (data->phase2_type == EAP_TYPE_NONE) {
-			int i;
+		if (data->phase2_type.vendor == EAP_VENDOR_IETF &&
+		    data->phase2_type.method == EAP_TYPE_NONE) {
+			size_t i;
 			for (i = 0; i < data->num_phase2_types; i++) {
-				if (data->phase2_types[i] != *pos)
+				if (data->phase2_types[i].vendor !=
+				    EAP_VENDOR_IETF ||
+				    data->phase2_types[i].method != *pos)
 					continue;
 
-				data->phase2_type = *pos;
+				data->phase2_type.vendor =
+					data->phase2_types[i].vendor;
+				data->phase2_type.method =
+					data->phase2_types[i].method;
 				wpa_printf(MSG_DEBUG, "EAP-FAST: Selected "
-					   "Phase 2 EAP method %d",
-					   data->phase2_type);
+					   "Phase 2 EAP vendor %d method %d",
+					   data->phase2_type.vendor,
+					   data->phase2_type.method);
 				break;
 			}
 		}
-		if (*pos != data->phase2_type || *pos == EAP_TYPE_NONE) {
+		if (*pos != data->phase2_type.method ||
+		    *pos == EAP_TYPE_NONE) {
 			if (eap_fast_phase2_nak(sm, data, hdr, resp, resp_len))
 				return -1;
 			return 0;
 		}
 
 		if (data->phase2_priv == NULL) {
-			data->phase2_method = eap_sm_get_eap_methods(*pos);
+			data->phase2_method = eap_sm_get_eap_methods(
+				data->phase2_type.vendor,
+				data->phase2_type.method);
 			if (data->phase2_method) {
 				if (data->key_block_p) {
 					sm->auth_challenge =
@@ -1464,10 +1487,10 @@ static int eap_fast_decrypt(struct eap_sm *sm, struct eap_fast_data *data,
 			    u8 **out_data, size_t *out_len)
 {
 	u8 *in_decrypted, *pos, *end;
-	int buf_len, len_decrypted, len;
+	int len_decrypted, len;
 	struct eap_hdr *hdr;
 	u8 *resp = NULL;
-	size_t resp_len;
+	size_t buf_len, resp_len;
 	int mandatory, tlv_type;
 	u8 *eap_payload_tlv = NULL, *pac = NULL;
 	size_t eap_payload_tlv_len = 0, pac_len = 0;
@@ -1926,15 +1949,7 @@ static int eap_fast_get_status(struct eap_sm *sm, void *priv, char *buf,
 			       size_t buflen, int verbose)
 {
 	struct eap_fast_data *data = priv;
-	int len;
-
-	len = eap_tls_status(sm, &data->ssl, buf, buflen, verbose);
-	if (data->phase2_method) {
-		len += snprintf(buf + len, buflen - len,
-				"EAP-FAST Phase2 method=%s\n",
-				data->phase2_method->name);
-	}
-	return len;
+	return eap_tls_status(sm, &data->ssl, buf, buflen, verbose);
 }
 
 
@@ -1966,17 +1981,23 @@ static u8 * eap_fast_getKey(struct eap_sm *sm, void *priv, size_t *len)
 
 const struct eap_method eap_method_fast =
 {
-	.method = EAP_TYPE_FAST,
-	.name = "FAST",
-	.init = eap_fast_init,
-	.deinit = eap_fast_deinit,
-	.process = eap_fast_process,
-	.isKeyAvailable = eap_fast_isKeyAvailable,
-	.getKey = eap_fast_getKey,
-	.get_status = eap_fast_get_status,
+	EAP_VENDOR_IETF,
+	EAP_TYPE_FAST,
+	"FAST",
+	eap_fast_init,
+	eap_fast_deinit,
+	eap_fast_process,
+	eap_fast_isKeyAvailable,
+	eap_fast_getKey,
+	eap_fast_get_status,
 #if 0
-	.has_reauth_data = eap_fast_has_reauth_data,
-	.deinit_for_reauth = eap_fast_deinit_for_reauth,
-	.init_for_reauth = eap_fast_init_for_reauth,
+	eap_fast_has_reauth_data,
+	eap_fast_deinit_for_reauth,
+	eap_fast_init_for_reauth,
+#else
+	NULL /* has_reauth_data */,
+	NULL /* deinit_for_reauth */,
+	NULL /* init_for_reauth */,
 #endif
+	NULL /* get_identity */
 };
