@@ -12,8 +12,7 @@
  * See README and COPYING for more details.
  */
 
-#include <stdlib.h>
-#include <stdio.h>
+#include "includes.h"
 
 #include "common.h"
 #include "eap_i.h"
@@ -44,8 +43,7 @@ static u8 * eap_md5_process(struct eap_sm *sm, void *priv,
 	struct eap_hdr *resp;
 	const u8 *pos, *challenge;
 	u8 *rpos;
-	int challenge_len;
-	size_t len;
+	size_t len, challenge_len;
 	const u8 *addr[3];
 	size_t elen[3];
 
@@ -56,10 +54,9 @@ static u8 * eap_md5_process(struct eap_sm *sm, void *priv,
 		return NULL;
 	}
 
-	pos = eap_hdr_validate(EAP_TYPE_MD5, reqData, reqDataLen, &len);
-	if (pos == NULL || len == 0) {
-		wpa_printf(MSG_INFO, "EAP-MD5: Invalid frame (pos=%p len=%lu)",
-			   pos, (unsigned long) len);
+	pos = eap_hdr_validate(EAP_VENDOR_IETF, EAP_TYPE_MD5,
+			       reqData, reqDataLen, &len);
+	if (pos == NULL) {
 		ret->ignore = TRUE;
 		return NULL;
 	}
@@ -68,8 +65,8 @@ static u8 * eap_md5_process(struct eap_sm *sm, void *priv,
 	if (challenge_len == 0 ||
 	    challenge_len > len - 1) {
 		wpa_printf(MSG_INFO, "EAP-MD5: Invalid challenge "
-			   "(challenge_len=%d len=%lu)",
-			   challenge_len, (unsigned long) len);
+			   "(challenge_len=%lu len=%lu",
+			   (unsigned long) challenge_len, (unsigned long) len);
 		ret->ignore = TRUE;
 		return NULL;
 	}
@@ -109,9 +106,17 @@ static u8 * eap_md5_process(struct eap_sm *sm, void *priv,
 
 const struct eap_method eap_method_md5 =
 {
-	.method = EAP_TYPE_MD5,
-	.name = "MD5",
-	.init = eap_md5_init,
-	.deinit = eap_md5_deinit,
-	.process = eap_md5_process,
+	EAP_VENDOR_IETF,
+	EAP_TYPE_MD5,
+	"MD5",
+	eap_md5_init,
+	eap_md5_deinit,
+	eap_md5_process,
+	NULL /* isKeyAvailable */,
+	NULL /* getKey */,
+	NULL /* get_status */,
+	NULL /* has_reauth_data */,
+	NULL /* deinit_for_reauth */,
+	NULL /* init_for_reauth */,
+	NULL /* get_identity */
 };
