@@ -12,9 +12,7 @@
  * See README and COPYING for more details.
  */
 
-#include <stdlib.h>
-#include <stdio.h>
-#include <string.h>
+#include "includes.h"
 
 #include "common.h"
 #include "eap_i.h"
@@ -22,16 +20,30 @@
 #include "wpa_supplicant.h"
 #include "config_ssid.h"
 #include "ms_funcs.h"
+#include "sha1.h"
 #include "crypto.h"
 #include "tls.h"
 #include "eap_ttls.h"
 
 
+/* Maximum supported PEAP version
+ * 0 = draft-ietf-pppext-eap-ttls-03.txt / draft-funk-eap-ttls-v0-00.txt
+ * 1 = draft-funk-eap-ttls-v1-00.txt
+ */
+#define EAP_TTLS_VERSION 0 /* TTLSv1 implementation is not yet complete */
+
+
+#define MSCHAPV2_KEY_LEN 16
+
+
 static void eap_ttls_deinit(struct eap_sm *sm, void *priv);
 
 
 struct eap_ttls_data {
 	struct eap_ssl_data ssl;
+	int ssl_initialized;
+
+	int ttls_version, force_ttls_version;
 
 	const struct eap_method *phase2_method;
 	void *phase2_priv;
@@ -45,8 +57,8 @@ struct eap_ttls_data {
 		EAP_TTLS_PHASE2_PAP,
 		EAP_TTLS_PHASE2_CHAP
 	} phase2_type;
-	u8 phase2_eap_type;
-	u8 *phase2_eap_types;
+	struct eap_method_type phase2_eap_type;
+	struct eap_method_type *phase2_eap_types;
 	size_t num_phase2_eap_types;
 
 	u8 auth_response[20];
@@ -71,8 +83,21 @@ static void * eap_ttls_init(struct eap_sm *sm)
 	if (data == NULL)
 		return NULL;
 	memset(data, 0, sizeof(*data));
+	data->ttls_version = EAP_TTLS_VERSION;
+	data->force_ttls_version = -1;
 	selected = "EAP";
 	data->phase2_type = EAP_TTLS_PHASE2_EAP;
+
+	if (config && config->phase1) {
+		char *pos = strstr(config->phase1, "ttlsver=");
+		if (pos) {
+			data->force_ttls_version = atoi(pos + 8);
+			data->ttls_version = data->force_ttls_version;
+			wpa_printf(MSG_DEBUG, "EAP-TTLS: Forced TTLS version "
+				   "%d", data->force_ttls_version);
+		}
+	}
+
 	if (config && config->phase2) {
 		if (strstr(config->phase2, "autheap=")) {
 			selected = "EAP";
@@ -96,7 +121,8 @@ static void * eap_ttls_init(struct eap_sm *sm)
 	if (data->phase2_type == EAP_TTLS_PHASE2_EAP) {
 		if (config && config->phase2) {
 			char *start, *pos, *buf;
-			u8 method, *methods = NULL, *_methods;
+			struct eap_method_type *methods = NULL, *_methods;
+			u8 method;
 			size_t num_methods = 0;
 			start = buf = strdup(config->phase2);
 			if (buf == NULL) {
@@ -104,6 +130,7 @@ static void * eap_ttls_init(struct eap_sm *sm)
 				return NULL;
 			}
 			while (start && *start != '\0') {
+				int vendor;
 				pos = strstr(start, "autheap=");
 				if (pos == NULL)
 					break;
@@ -116,23 +143,27 @@ static void * eap_ttls_init(struct eap_sm *sm)
 				pos = strchr(start, ' ');
 				if (pos)
 					*pos++ = '\0';
-				method = eap_get_phase2_type(start);
-				if (method == EAP_TYPE_NONE) {
+				method = eap_get_phase2_type(start, &vendor);
+				if (vendor == EAP_VENDOR_IETF &&
+				    method == EAP_TYPE_NONE) {
 					wpa_printf(MSG_ERROR, "EAP-TTLS: "
 						   "Unsupported Phase2 EAP "
 						   "method '%s'", start);
 				} else {
 					num_methods++;
-					_methods = realloc(methods,
-							   num_methods);
+					_methods = realloc(
+						methods, num_methods *
+						sizeof(*methods));
 					if (_methods == NULL) {
 						free(methods);
-						free(buf);
 						eap_ttls_deinit(sm, data);
 						return NULL;
 					}
 					methods = _methods;
-					methods[num_methods - 1] = method;
+					methods[num_methods - 1].vendor =
+						vendor;
+					methods[num_methods - 1].method =
+						method;
 				}
 
 				start = pos;
@@ -152,16 +183,23 @@ static void * eap_ttls_init(struct eap_sm *sm)
 			return NULL;
 		}
 		wpa_hexdump(MSG_DEBUG, "EAP-TTLS: Phase2 EAP types",
-			    data->phase2_eap_types,
-			    data->num_phase2_eap_types);
-		data->phase2_eap_type = EAP_TYPE_NONE;
+			    (u8 *) data->phase2_eap_types,
+			    data->num_phase2_eap_types *
+			    sizeof(struct eap_method_type));
+		data->phase2_eap_type.vendor = EAP_VENDOR_IETF;
+		data->phase2_eap_type.method = EAP_TYPE_NONE;
 	}
 
-
-	if (eap_tls_ssl_init(sm, &data->ssl, config)) {
-		wpa_printf(MSG_INFO, "EAP-TTLS: Failed to initialize SSL.");
-		eap_ttls_deinit(sm, data);
-		return NULL;
+	if (!(tls_capabilities(sm->ssl_ctx) & TLS_CAPABILITY_IA) &&
+	    data->ttls_version > 0) {
+		if (data->force_ttls_version > 0) {
+			wpa_printf(MSG_INFO, "EAP-TTLS: Forced TTLSv%d and "
+				   "TLS library does not support TLS/IA.",
+				   data->force_ttls_version);
+			eap_ttls_deinit(sm, data);
+			return NULL;
+		}
+		data->ttls_version = 0;
 	}
 
 	return data;
@@ -176,7 +214,8 @@ static void eap_ttls_deinit(struct eap_sm *sm, void *priv)
 	if (data->phase2_priv && data->phase2_method)
 		data->phase2_method->deinit(sm, data->phase2_priv);
 	free(data->phase2_eap_types);
-	eap_tls_ssl_deinit(sm, &data->ssl);
+	if (data->ssl_initialized)
+		eap_tls_ssl_deinit(sm, &data->ssl);
 	free(data->key_data);
 	free(data->pending_phase2_req);
 	free(data);
@@ -202,7 +241,7 @@ static int eap_ttls_encrypt(struct eap_sm *sm, struct eap_ttls_data *data,
 
 	pos = (u8 *) (resp + 1);
 	*pos++ = EAP_TYPE_TTLS;
-	*pos++ = 0;
+	*pos++ = data->ttls_version;
 
 	res = tls_connection_encrypt(sm->ssl_ctx, data->ssl.conn,
 				     plain, plain_len,
@@ -282,6 +321,166 @@ static int eap_ttls_avp_encapsulate(u8 **resp, size_t *resp_len, u32 avp_code,
 }
 
 
+static int eap_ttls_ia_permute_inner_secret(struct eap_sm *sm,
+					    struct eap_ttls_data *data,
+					    const u8 *key, size_t key_len)
+{
+	u8 *buf;
+	size_t buf_len;
+	int ret;
+
+	if (key) {
+		buf_len = 2 + key_len;
+		buf = malloc(buf_len);
+		if (buf == NULL)
+			return -1;
+		WPA_PUT_BE16(buf, key_len);
+		memcpy(buf + 2, key, key_len);
+	} else {
+		buf = NULL;
+		buf_len = 0;
+	}
+
+	wpa_hexdump_key(MSG_DEBUG, "EAP-TTLS: Session keys for TLS/IA inner "
+			"secret permutation", buf, buf_len);
+	ret = tls_connection_ia_permute_inner_secret(sm->ssl_ctx,
+						     data->ssl.conn,
+						     buf, buf_len);
+	free(buf);
+
+	return ret;
+}
+
+
+static int eap_ttls_v0_derive_key(struct eap_sm *sm,
+				  struct eap_ttls_data *data)
+{
+	free(data->key_data);
+	data->key_data = eap_tls_derive_key(sm, &data->ssl,
+					    "ttls keying material",
+					    EAP_TLS_KEY_LEN);
+	if (!data->key_data) {
+		wpa_printf(MSG_INFO, "EAP-TTLS: Failed to derive key");
+		return -1;
+	}
+
+	wpa_hexdump_key(MSG_DEBUG, "EAP-TTLS: Derived key",
+			data->key_data, EAP_TLS_KEY_LEN);
+
+	return 0;
+}
+
+
+static int eap_ttls_v1_derive_key(struct eap_sm *sm,
+				  struct eap_ttls_data *data)
+{
+	struct tls_keys keys;
+	u8 *rnd;
+
+	free(data->key_data);
+	data->key_data = NULL;
+
+	memset(&keys, 0, sizeof(keys));
+	if (tls_connection_get_keys(sm->ssl_ctx, data->ssl.conn, &keys) ||
+	    keys.client_random == NULL || keys.server_random == NULL ||
+	    keys.inner_secret == NULL) {
+		wpa_printf(MSG_INFO, "EAP-TTLS: Could not get inner secret, "
+			   "client random, or server random to derive keying "
+			   "material");
+		return -1;
+	}
+
+	rnd = malloc(keys.client_random_len + keys.server_random_len);
+	data->key_data = malloc(EAP_TLS_KEY_LEN);
+	if (rnd == NULL || data->key_data == NULL) {
+		wpa_printf(MSG_INFO, "EAP-TTLS: No memory for key derivation");
+		free(rnd);
+		free(data->key_data);
+		data->key_data = NULL;
+		return -1;
+	}
+	memcpy(rnd, keys.client_random, keys.client_random_len);
+	memcpy(rnd + keys.client_random_len, keys.server_random,
+	       keys.server_random_len);
+
+	if (tls_prf(keys.inner_secret, keys.inner_secret_len,
+		    "ttls v1 keying material", rnd, keys.client_random_len +
+		    keys.server_random_len, data->key_data, EAP_TLS_KEY_LEN)) {
+		wpa_printf(MSG_DEBUG, "EAP-TTLS: Failed to derive key");
+		free(rnd);
+		free(data->key_data);
+		data->key_data = NULL;
+		return -1;
+	}
+
+	wpa_hexdump(MSG_DEBUG, "EAP-TTLS: client/server random",
+		    rnd, keys.client_random_len + keys.server_random_len);
+	wpa_hexdump_key(MSG_DEBUG, "EAP-TTLS: TLS/IA inner secret",
+			keys.inner_secret, keys.inner_secret_len);
+
+	free(rnd);
+
+	wpa_hexdump_key(MSG_DEBUG, "EAP-TTLS: Derived key",
+			data->key_data, EAP_TLS_KEY_LEN);
+
+	return 0;
+}
+
+
+static u8 * eap_ttls_implicit_challenge(struct eap_sm *sm,
+					struct eap_ttls_data *data, size_t len)
+{
+	struct tls_keys keys;
+	u8 *challenge, *rnd;
+
+	if (data->ttls_version == 0) {
+		return eap_tls_derive_key(sm, &data->ssl, "ttls challenge",
+					  len);
+	}
+
+	memset(&keys, 0, sizeof(keys));
+	if (tls_connection_get_keys(sm->ssl_ctx, data->ssl.conn, &keys) ||
+	    keys.client_random == NULL || keys.server_random == NULL ||
+	    keys.inner_secret == NULL) {
+		wpa_printf(MSG_INFO, "EAP-TTLS: Could not get inner secret, "
+			   "client random, or server random to derive "
+			   "implicit challenge");
+		return NULL;
+	}
+
+	rnd = malloc(keys.client_random_len + keys.server_random_len);
+	challenge = malloc(len);
+	if (rnd == NULL || challenge == NULL) {
+		wpa_printf(MSG_INFO, "EAP-TTLS: No memory for implicit "
+			   "challenge derivation");
+		free(rnd);
+		free(challenge);
+		return NULL;
+	}
+	memcpy(rnd, keys.server_random, keys.server_random_len);
+	memcpy(rnd + keys.server_random_len, keys.client_random,
+	       keys.client_random_len);
+
+	if (tls_prf(keys.inner_secret, keys.inner_secret_len,
+		    "inner application challenge", rnd,
+		    keys.client_random_len + keys.server_random_len,
+		    challenge, len)) {
+		wpa_printf(MSG_DEBUG, "EAP-TTLS: Failed to derive implicit "
+			   "challenge");
+		free(rnd);
+		free(challenge);
+		return NULL;
+	}
+
+	free(rnd);
+
+	wpa_hexdump_key(MSG_DEBUG, "EAP-TTLS: Derived implicit challenge",
+			challenge, len);
+
+	return challenge;
+}
+
+
 static int eap_ttls_phase2_nak(struct eap_sm *sm,
 			       struct eap_ttls_data *data,
 			       struct eap_hdr *hdr,
@@ -292,7 +491,8 @@ static int eap_ttls_phase2_nak(struct eap_sm *sm,
 
 	wpa_printf(MSG_DEBUG, "EAP-TTLS: Phase 2 Request: Nak type=%d", *pos);
 	wpa_hexdump(MSG_DEBUG, "EAP-TTLS: Allowed Phase2 EAP types",
-		    data->phase2_eap_types, data->num_phase2_eap_types);
+		    (u8 *) data->phase2_eap_types, data->num_phase2_eap_types *
+		    sizeof(struct eap_method_type));
 	*resp_len = sizeof(struct eap_hdr) + 1 + data->num_phase2_eap_types;
 	*resp = malloc(*resp_len);
 	if (*resp == NULL)
@@ -334,32 +534,43 @@ static int eap_ttls_phase2_request_eap(struct eap_sm *sm,
 		*resp = eap_sm_buildIdentity(sm, req->identifier, resp_len, 1);
 		break;
 	default:
-		if (data->phase2_eap_type == EAP_TYPE_NONE) {
-			int i;
+		if (data->phase2_eap_type.vendor == EAP_VENDOR_IETF &&
+		    data->phase2_eap_type.method == EAP_TYPE_NONE) {
+			size_t i;
 			for (i = 0; i < data->num_phase2_eap_types; i++) {
-				if (data->phase2_eap_types[i] != *pos)
+				if (data->phase2_eap_types[i].vendor !=
+				    EAP_VENDOR_IETF ||
+				    data->phase2_eap_types[i].method != *pos)
 					continue;
 
-				data->phase2_eap_type = *pos;
+				data->phase2_eap_type.vendor =
+					data->phase2_eap_types[i].vendor;
+				data->phase2_eap_type.method =
+					data->phase2_eap_types[i].method;
 				wpa_printf(MSG_DEBUG, "EAP-TTLS: Selected "
-					   "Phase 2 EAP method %d",
-					   data->phase2_eap_type);
+					   "Phase 2 EAP vendor %d method %d",
+					   data->phase2_eap_type.vendor,
+					   data->phase2_eap_type.method);
 				break;
 			}
 		}
-		if (*pos != data->phase2_eap_type || *pos == EAP_TYPE_NONE) {
+		if (*pos != data->phase2_eap_type.method ||
+		    *pos == EAP_TYPE_NONE) {
 			if (eap_ttls_phase2_nak(sm, data, hdr, resp, resp_len))
 				return -1;
 			break;
 		}
 
 		if (data->phase2_priv == NULL) {
-			data->phase2_method = eap_sm_get_eap_methods(*pos);
+			data->phase2_method = eap_sm_get_eap_methods(
+				EAP_VENDOR_IETF, *pos);
 			if (data->phase2_method) {
 				sm->init_phase2 = 1;
+				sm->mschapv2_full_key = 1;
 				data->phase2_priv =
 					data->phase2_method->init(sm);
 				sm->init_phase2 = 0;
+				sm->mschapv2_full_key = 0;
 			}
 		}
 		if (data->phase2_priv == NULL || data->phase2_method == NULL) {
@@ -379,6 +590,28 @@ static int eap_ttls_phase2_request_eap(struct eap_sm *sm,
 			ret->methodState = iret.methodState;
 			ret->decision = iret.decision;
 		}
+		if (data->ttls_version > 0) {
+			const struct eap_method *m = data->phase2_method;
+			void *priv = data->phase2_priv;
+
+			/* TTLSv1 requires TLS/IA FinalPhaseFinished */
+			if (ret->decision == DECISION_UNCOND_SUCC)
+				ret->decision = DECISION_COND_SUCC;
+			ret->methodState = METHOD_CONT;
+
+			if (ret->decision == DECISION_COND_SUCC &&
+			    m->isKeyAvailable && m->getKey &&
+			    m->isKeyAvailable(sm, priv)) {
+				u8 *key;
+				size_t key_len;
+				key = m->getKey(sm, priv, &key_len);
+				if (key) {
+					eap_ttls_ia_permute_inner_secret(
+						sm, data, key, key_len);
+					free(key);
+				}
+			}
+		}
 		break;
 	}
 
@@ -407,8 +640,7 @@ static int eap_ttls_phase2_request_mschapv2(struct eap_sm *sm,
 {
 	struct wpa_ssid *config = eap_get_config(sm);
 	u8 *buf, *pos, *challenge, *username, *peer_challenge;
-	size_t username_len;
-	int i;
+	size_t username_len, i;
 
 	wpa_printf(MSG_DEBUG, "EAP-TTLS: Phase 2 MSCHAPV2 Request");
 
@@ -438,9 +670,8 @@ static int eap_ttls_phase2_request_mschapv2(struct eap_sm *sm,
 			       config->identity, config->identity_len);
 
 	/* MS-CHAP-Challenge */
-	challenge = eap_tls_derive_key(sm, &data->ssl, "ttls challenge",
-				       EAP_TTLS_MSCHAPV2_CHALLENGE_LEN * 2 +
-				       1);
+	challenge = eap_ttls_implicit_challenge(
+		sm, data, EAP_TTLS_MSCHAPV2_CHALLENGE_LEN * 2 + 1);
 	if (challenge == NULL) {
 		free(buf);
 		wpa_printf(MSG_ERROR, "EAP-TTLS/MSCHAPV2: Failed to derive "
@@ -483,6 +714,24 @@ static int eap_ttls_phase2_request_mschapv2(struct eap_sm *sm,
 					pos, data->auth_response);
 	data->auth_response_valid = 1;
 
+	if (data->ttls_version > 0) {
+		u8 pw_hash[16], pw_hash_hash[16], master_key[16];
+		u8 session_key[2 * MSCHAPV2_KEY_LEN];
+		nt_password_hash(config->password, config->password_len,
+				 pw_hash);
+		hash_nt_password_hash(pw_hash, pw_hash_hash);
+		get_master_key(pw_hash_hash, pos /* nt_response */,
+			       master_key);
+		get_asymetric_start_key(master_key, session_key,
+					MSCHAPV2_KEY_LEN, 0, 0);
+		get_asymetric_start_key(master_key,
+					session_key + MSCHAPV2_KEY_LEN,
+					MSCHAPV2_KEY_LEN, 1, 0);
+		eap_ttls_ia_permute_inner_secret(sm, data,
+						 session_key,
+						 sizeof(session_key));
+	}
+
 	pos += 24;
 	free(challenge);
 	AVP_PAD(buf, pos);
@@ -490,7 +739,7 @@ static int eap_ttls_phase2_request_mschapv2(struct eap_sm *sm,
 	*resp = buf;
 	*resp_len = pos - buf;
 
-	if (sm->workaround) {
+	if (sm->workaround && data->ttls_version == 0) {
 		/* At least FreeRADIUS seems to be terminating
 		 * EAP-TTLS/MSHCAPV2 without the expected MS-CHAP-v2 Success
 		 * packet. */
@@ -528,8 +777,7 @@ static int eap_ttls_phase2_request_mschap(struct eap_sm *sm,
 			       config->identity, config->identity_len);
 
 	/* MS-CHAP-Challenge */
-	challenge = eap_tls_derive_key(sm, &data->ssl, "ttls challenge",
-				       EAP_TLS_KEY_LEN);
+	challenge = eap_ttls_implicit_challenge(sm, data, EAP_TLS_KEY_LEN);
 	if (challenge == NULL) {
 		free(buf);
 		wpa_printf(MSG_ERROR, "EAP-TTLS/MSCHAP: Failed to derive "
@@ -565,10 +813,17 @@ static int eap_ttls_phase2_request_mschap(struct eap_sm *sm,
 	*resp = buf;
 	*resp_len = pos - buf;
 
-	/* EAP-TTLS/MSCHAP does not provide tunneled success notification, so
-	 * assume that Phase2 succeeds. */
-	ret->methodState = METHOD_DONE;
-	ret->decision = DECISION_COND_SUCC;
+	if (data->ttls_version > 0) {
+		/* EAP-TTLSv1 uses TLS/IA FinalPhaseFinished to report success,
+		 * so do not allow connection to be terminated yet. */
+		ret->methodState = METHOD_CONT;
+		ret->decision = DECISION_COND_SUCC;
+	} else {
+		/* EAP-TTLS/MSCHAP does not provide tunneled success
+		 * notification, so assume that Phase2 succeeds. */
+		ret->methodState = METHOD_DONE;
+		ret->decision = DECISION_COND_SUCC;
+	}
 
 	return 0;
 }
@@ -613,10 +868,17 @@ static int eap_ttls_phase2_request_pap(struct eap_sm *sm,
 	*resp = buf;
 	*resp_len = pos - buf;
 
-	/* EAP-TTLS/PAP does not provide tunneled success notification, so
-	 * assume that Phase2 succeeds. */
-	ret->methodState = METHOD_DONE;
-	ret->decision = DECISION_COND_SUCC;
+	if (data->ttls_version > 0) {
+		/* EAP-TTLSv1 uses TLS/IA FinalPhaseFinished to report success,
+		 * so do not allow connection to be terminated yet. */
+		ret->methodState = METHOD_CONT;
+		ret->decision = DECISION_COND_SUCC;
+	} else {
+		/* EAP-TTLS/PAP does not provide tunneled success notification,
+		 * so assume that Phase2 succeeds. */
+		ret->methodState = METHOD_DONE;
+		ret->decision = DECISION_COND_SUCC;
+	}
 
 	return 0;
 }
@@ -648,8 +910,7 @@ static int eap_ttls_phase2_request_chap(struct eap_sm *sm,
 			       config->identity, config->identity_len);
 
 	/* CHAP-Challenge */
-	challenge = eap_tls_derive_key(sm, &data->ssl, "ttls challenge",
-				       EAP_TLS_KEY_LEN);
+	challenge = eap_ttls_implicit_challenge(sm, data, EAP_TLS_KEY_LEN);
 	if (challenge == NULL) {
 		free(buf);
 		wpa_printf(MSG_ERROR, "EAP-TTLS/CHAP: Failed to derive "
@@ -690,10 +951,17 @@ static int eap_ttls_phase2_request_chap(struct eap_sm *sm,
 	*resp = buf;
 	*resp_len = pos - buf;
 
-	/* EAP-TTLS/CHAP does not provide tunneled success notification, so
-	 * assume that Phase2 succeeds. */
-	ret->methodState = METHOD_DONE;
-	ret->decision = DECISION_COND_SUCC;
+	if (data->ttls_version > 0) {
+		/* EAP-TTLSv1 uses TLS/IA FinalPhaseFinished to report success,
+		 * so do not allow connection to be terminated yet. */
+		ret->methodState = METHOD_CONT;
+		ret->decision = DECISION_COND_SUCC;
+	} else {
+		/* EAP-TTLS/CHAP does not provide tunneled success
+		 * notification, so assume that Phase2 succeeds. */
+		ret->methodState = METHOD_DONE;
+		ret->decision = DECISION_COND_SUCC;
+	}
 
 	return 0;
 }
@@ -713,7 +981,7 @@ static int eap_ttls_phase2_request(struct eap_sm *sm,
 	    data->phase2_type == EAP_TTLS_PHASE2_MSCHAP ||
 	    data->phase2_type == EAP_TTLS_PHASE2_PAP ||
 	    data->phase2_type == EAP_TTLS_PHASE2_CHAP) {
-		if (config->identity == NULL) {
+		if (config == NULL || config->identity == NULL) {
 			wpa_printf(MSG_INFO,
 				   "EAP-TTLS: Identity not configured");
 			eap_sm_request_identity(sm, config);
@@ -766,6 +1034,43 @@ static int eap_ttls_phase2_request(struct eap_sm *sm,
 }
 
 
+static u8 * eap_ttls_build_phase_finished(struct eap_sm *sm,
+					  struct eap_ttls_data *data,
+					  int id, int final,
+					  size_t *reqDataLen)
+{
+	int len;
+	struct eap_hdr *req;
+	u8 *pos;
+	const int max_len = 300;
+
+	len = sizeof(struct eap_hdr) + 2 + max_len;
+	req = malloc(len);
+	if (req == NULL)
+		return NULL;
+
+	req->code = EAP_CODE_RESPONSE;
+	req->identifier = id;
+
+	pos = (u8 *) (req + 1);
+	*pos++ = EAP_TYPE_TTLS;
+	*pos++ = data->ttls_version;
+
+	len = tls_connection_ia_send_phase_finished(sm->ssl_ctx,
+						    data->ssl.conn,
+						    final, pos, max_len);
+	if (len < 0) {
+		free(req);
+		return NULL;
+	}
+
+	*reqDataLen = sizeof(struct eap_hdr) + 2 + len;
+	req->length = host_to_be16(*reqDataLen);
+
+	return (u8 *) req;
+}
+
+
 static int eap_ttls_decrypt(struct eap_sm *sm, struct eap_ttls_data *data,
 			    struct eap_method_ret *ret,
 			    const struct eap_hdr *req,
@@ -773,10 +1078,10 @@ static int eap_ttls_decrypt(struct eap_sm *sm, struct eap_ttls_data *data,
 			    u8 **out_data, size_t *out_len)
 {
 	u8 *in_decrypted = NULL, *pos;
-	int buf_len, len_decrypted = 0, len, left, retval = 0;
+	int res, retval = 0;
 	struct eap_hdr *hdr = NULL;
 	u8 *resp = NULL, *mschapv2 = NULL, *eapdata = NULL;
-	size_t resp_len, eap_len = 0;
+	size_t resp_len, eap_len = 0, len_decrypted = 0, len, buf_len, left;
 	struct ttls_avp *avp;
 	u8 recv_response[20];
 	int mschapv2_error = 0;
@@ -866,18 +1171,34 @@ static int eap_ttls_decrypt(struct eap_sm *sm, struct eap_ttls_data *data,
 		goto done;
 	}
 
-	len_decrypted = tls_connection_decrypt(sm->ssl_ctx, data->ssl.conn,
-					       msg, msg_len,
-					       in_decrypted, buf_len);
+	res = tls_connection_decrypt(sm->ssl_ctx, data->ssl.conn,
+				     msg, msg_len, in_decrypted, buf_len);
 	free(data->ssl.tls_in);
 	data->ssl.tls_in = NULL;
 	data->ssl.tls_in_len = 0;
-	if (len_decrypted < 0) {
+	if (res < 0) {
 		wpa_printf(MSG_INFO, "EAP-TTLS: Failed to decrypt Phase 2 "
 			   "data");
 		retval = -1;
 		goto done;
 	}
+	len_decrypted = res;
+
+	if (data->ttls_version > 0 && len_decrypted == 0 &&
+	    tls_connection_ia_final_phase_finished(sm->ssl_ctx,
+						   data->ssl.conn)) {
+		wpa_printf(MSG_DEBUG, "EAP-TTLS: FinalPhaseFinished received");
+		wpa_printf(MSG_INFO, "EAP-TTLS: TLS/IA authentication "
+			   "succeeded");
+		ret->methodState = METHOD_DONE;
+		ret->decision = DECISION_UNCOND_SUCC;
+		data->phase2_success = 1;
+		*out_data = eap_ttls_build_phase_finished(sm, data,
+							  req->identifier, 1,
+							  out_len);
+		eap_ttls_v1_derive_key(sm, data);
+		goto done;
+	}
 
 continue_req:
 	data->phase2_start = 0;
@@ -886,8 +1207,8 @@ continue_req:
 		    in_decrypted, len_decrypted);
 	if (len_decrypted < sizeof(struct ttls_avp)) {
 		wpa_printf(MSG_WARNING, "EAP-TTLS: Too short Phase 2 AVP frame"
-			   " len=%d expected %lu or more - dropped",
-			   len_decrypted,
+			   " len=%lu expected %lu or more - dropped",
+			   (unsigned long) len_decrypted,
 			   (unsigned long) sizeof(struct ttls_avp));
 		retval = -1;
 		goto done;
@@ -912,8 +1233,8 @@ continue_req:
 			   (int) avp_length);
 		if (avp_length > left) {
 			wpa_printf(MSG_WARNING, "EAP-TTLS: AVP overflow "
-				   "(len=%d, left=%d) - dropped",
-				   (int) avp_length, left);
+				   "(len=%d, left=%lu) - dropped",
+				   (int) avp_length, (unsigned long) left);
 			retval = -1;
 			goto done;
 		}
@@ -1000,7 +1321,10 @@ continue_req:
 
 		pad = (4 - (avp_length & 3)) & 3;
 		pos += avp_length + pad;
-		left -= avp_length + pad;
+		if (left < avp_length + pad)
+			left = 0;
+		else
+			left -= avp_length + pad;
 	}
 
 	switch (data->phase2_type) {
@@ -1027,15 +1351,16 @@ continue_req:
 		len = be_to_host16(hdr->length);
 		if (len > eap_len) {
 			wpa_printf(MSG_INFO, "EAP-TTLS: Length mismatch in "
-				   "Phase 2 EAP frame (EAP hdr len=%d, EAP "
-				   "data len in AVP=%lu)", len,
+				   "Phase 2 EAP frame (EAP hdr len=%lu, EAP "
+				   "data len in AVP=%lu)",
+				   (unsigned long) len,
 				   (unsigned long) eap_len);
 			retval = -1;
 			goto done;
 		}
 		wpa_printf(MSG_DEBUG, "EAP-TTLS: received Phase 2: code=%d "
-			   "identifier=%d length=%d",
-			   hdr->code, hdr->identifier, len);
+			   "identifier=%d length=%lu",
+			   hdr->code, hdr->identifier, (unsigned long) len);
 	process_eap:
 		switch (hdr->code) {
 		case EAP_CODE_REQUEST:
@@ -1093,9 +1418,17 @@ continue_req:
 
 		wpa_printf(MSG_INFO, "EAP-TTLS: Phase 2 MSCHAPV2 "
 			   "authentication succeeded");
-		ret->methodState = METHOD_DONE;
-		ret->decision = DECISION_UNCOND_SUCC;
-		data->phase2_success = 1;
+		if (data->ttls_version > 0) {
+			/* EAP-TTLSv1 uses TLS/IA FinalPhaseFinished to report
+			 * success, so do not allow connection to be terminated
+			 * yet. */
+			ret->methodState = METHOD_CONT;
+			ret->decision = DECISION_COND_SUCC;
+		} else {
+			ret->methodState = METHOD_DONE;
+			ret->decision = DECISION_UNCOND_SUCC;
+			data->phase2_success = 1;
+		}
 
 		/* Reply with empty data; authentication server will reply
 		 * with EAP-Success after this. */
@@ -1159,6 +1492,7 @@ static u8 * eap_ttls_process(struct eap_sm *sm, void *priv,
 	u8 flags, *resp, id;
 	const u8 *pos;
 	struct eap_ttls_data *data = priv;
+	struct wpa_ssid *config = eap_get_config(sm);
 
 	pos = eap_tls_process_init(sm, &data->ssl, EAP_TYPE_TTLS, ret,
 				   reqData, reqDataLen, &left, &flags);
@@ -1168,6 +1502,34 @@ static u8 * eap_ttls_process(struct eap_sm *sm, void *priv,
 	id = req->identifier;
 
 	if (flags & EAP_TLS_FLAGS_START) {
+		wpa_printf(MSG_DEBUG, "EAP-TTLS: Start (server ver=%d, own "
+			   "ver=%d)", flags & EAP_PEAP_VERSION_MASK,
+			   data->ttls_version);
+		if ((flags & EAP_PEAP_VERSION_MASK) < data->ttls_version)
+			data->ttls_version = flags & EAP_PEAP_VERSION_MASK;
+		if (data->force_ttls_version >= 0 &&
+		    data->force_ttls_version != data->ttls_version) {
+			wpa_printf(MSG_WARNING, "EAP-TTLS: Failed to select "
+				   "forced TTLS version %d",
+				   data->force_ttls_version);
+			ret->methodState = METHOD_DONE;
+			ret->decision = DECISION_FAIL;
+			ret->allowNotifications = FALSE;
+			return NULL;
+		}
+		wpa_printf(MSG_DEBUG, "EAP-TTLS: Using TTLS version %d",
+			   data->ttls_version);
+
+		if (data->ttls_version > 0)
+			data->ssl.tls_ia = 1;
+		if (eap_tls_ssl_init(sm, &data->ssl, config)) {
+			wpa_printf(MSG_INFO, "EAP-TTLS: Failed to initialize "
+				   "SSL.");
+			eap_ttls_deinit(sm, data);
+			return NULL;
+		}
+		data->ssl_initialized = 1;
+
 		wpa_printf(MSG_DEBUG, "EAP-TTLS: Start");
 		/* draft-ietf-pppext-eap-ttls-03.txt, Ch. 8.1:
 		 * EAP-TTLS Start packet may, in a future specification, be
@@ -1175,6 +1537,13 @@ static u8 * eap_ttls_process(struct eap_sm *sm, void *priv,
 		 * must ignore such data but must not reject the Start packet.
 		 */
 		left = 0;
+	} else if (!data->ssl_initialized) {
+		wpa_printf(MSG_DEBUG, "EAP-TTLS: First message did not "
+			   "include Start flag");
+		ret->methodState = METHOD_DONE;
+		ret->decision = DECISION_FAIL;
+		ret->allowNotifications = FALSE;
+		return NULL;
 	}
 
 	resp = NULL;
@@ -1183,8 +1552,8 @@ static u8 * eap_ttls_process(struct eap_sm *sm, void *priv,
 		res = eap_ttls_decrypt(sm, data, ret, req, pos, left,
 				       &resp, respDataLen);
 	} else {
-		res = eap_tls_process_helper(sm, &data->ssl, EAP_TYPE_TTLS, 0,
-					     id, pos, left,
+		res = eap_tls_process_helper(sm, &data->ssl, EAP_TYPE_TTLS,
+					     data->ttls_version, id, pos, left,
 					     &resp, respDataLen);
 
 		if (tls_connection_established(sm->ssl_ctx, data->ssl.conn)) {
@@ -1197,20 +1566,8 @@ static u8 * eap_ttls_process(struct eap_sm *sm, void *priv,
 				ret->methodState = METHOD_MAY_CONT;
 			}
 			data->phase2_start = 1;
-			free(data->key_data);
-			data->key_data =
-				eap_tls_derive_key(sm, &data->ssl,
-						   "ttls keying material",
-						   EAP_TLS_KEY_LEN);
-			if (data->key_data) {
-				wpa_hexdump_key(MSG_DEBUG,
-						"EAP-TTLS: Derived key",
-						data->key_data,
-						EAP_TLS_KEY_LEN);
-			} else {
-				wpa_printf(MSG_DEBUG, "EAP-TTLS: Failed to "
-					   "derive key");
-			}
+			if (data->ttls_version == 0)
+				eap_ttls_v0_derive_key(sm, data);
 
 			if (*respDataLen == 0) {
 				if (eap_ttls_decrypt(sm, data, ret, req, NULL,
@@ -1225,7 +1582,7 @@ static u8 * eap_ttls_process(struct eap_sm *sm, void *priv,
 		}
 	}
 
-	if (ret->methodState == METHOD_DONE) {
+	if (data->ttls_version == 0 && ret->methodState == METHOD_DONE) {
 		ret->allowNotifications = FALSE;
 		if (ret->decision == DECISION_UNCOND_SUCC ||
 		    ret->decision == DECISION_COND_SUCC) {
@@ -1233,7 +1590,8 @@ static u8 * eap_ttls_process(struct eap_sm *sm, void *priv,
 				   "completed successfully");
 			data->phase2_success = 1;
 		}
-	} else if (sm->workaround && ret->methodState == METHOD_MAY_CONT &&
+	} else if (data->ttls_version == 0 && sm->workaround &&
+		   ret->methodState == METHOD_MAY_CONT &&
 		   (ret->decision == DECISION_UNCOND_SUCC ||
 		    ret->decision == DECISION_COND_SUCC)) {
 			wpa_printf(MSG_DEBUG, "EAP-TTLS: Authentication "
@@ -1243,7 +1601,7 @@ static u8 * eap_ttls_process(struct eap_sm *sm, void *priv,
 
 	if (res == 1) {
 		return eap_tls_build_ack(&data->ssl, respDataLen, id,
-					 EAP_TYPE_TTLS, 0);
+					 EAP_TYPE_TTLS, data->ttls_version);
 	}
 	return resp;
 }
@@ -1289,28 +1647,25 @@ static int eap_ttls_get_status(struct eap_sm *sm, void *priv, char *buf,
 	int len;
 
 	len = eap_tls_status(sm, &data->ssl, buf, buflen, verbose);
+	len += snprintf(buf + len, buflen - len, "EAP-TTLSv%d Phase2 method=",
+			data->ttls_version);
 	switch (data->phase2_type) {
 	case EAP_TTLS_PHASE2_EAP:
-		len += snprintf(buf + len, buflen - len,
-				"EAP-TTLS Phase2 method=EAP-%s\n",
+		len += snprintf(buf + len, buflen - len, "EAP-%s\n",
 				data->phase2_method ? data->phase2_method->name
 				: "?");
 		break;
 	case EAP_TTLS_PHASE2_MSCHAPV2:
-		len += snprintf(buf + len, buflen - len,
-				"EAP-TTLS Phase2 method=MSCHAPV2\n");
+		len += snprintf(buf + len, buflen - len, "MSCHAPV2\n");
 		break;
 	case EAP_TTLS_PHASE2_MSCHAP:
-		len += snprintf(buf + len, buflen - len,
-				"EAP-TTLS Phase2 method=MSCHAP\n");
+		len += snprintf(buf + len, buflen - len, "MSCHAP\n");
 		break;
 	case EAP_TTLS_PHASE2_PAP:
-		len += snprintf(buf + len, buflen - len,
-				"EAP-TTLS Phase2 method=PAP\n");
+		len += snprintf(buf + len, buflen - len, "PAP\n");
 		break;
 	case EAP_TTLS_PHASE2_CHAP:
-		len += snprintf(buf + len, buflen - len,
-				"EAP-TTLS Phase2 method=CHAP\n");
+		len += snprintf(buf + len, buflen - len, "CHAP\n");
 		break;
 	}
 
@@ -1346,15 +1701,17 @@ static u8 * eap_ttls_getKey(struct eap_sm *sm, void *priv, size_t *len)
 
 const struct eap_method eap_method_ttls =
 {
-	.method = EAP_TYPE_TTLS,
-	.name = "TTLS",
-	.init = eap_ttls_init,
-	.deinit = eap_ttls_deinit,
-	.process = eap_ttls_process,
-	.isKeyAvailable = eap_ttls_isKeyAvailable,
-	.getKey = eap_ttls_getKey,
-	.get_status = eap_ttls_get_status,
-	.has_reauth_data = eap_ttls_has_reauth_data,
-	.deinit_for_reauth = eap_ttls_deinit_for_reauth,
-	.init_for_reauth = eap_ttls_init_for_reauth,
+	EAP_VENDOR_IETF,
+	EAP_TYPE_TTLS,
+	"TTLS",
+	eap_ttls_init,
+	eap_ttls_deinit,
+	eap_ttls_process,
+	eap_ttls_isKeyAvailable,
+	eap_ttls_getKey,
+	eap_ttls_get_status,
+	eap_ttls_has_reauth_data,
+	eap_ttls_deinit_for_reauth,
+	eap_ttls_init_for_reauth,
+	NULL /* get_identity */
 };
