@@ -12,9 +12,7 @@
  * See README and COPYING for more details.
  */
 
-#include <stdlib.h>
-#include <stdio.h>
-#include <string.h>
+#include "includes.h"
 
 #include "common.h"
 #include "eapol_sm.h"
@@ -24,6 +22,10 @@
 #include "wpa.h"
 #include "md5.h"
 #include "rc4.h"
+#include "state_machine.h"
+
+#define STATE_MACHINE_DATA struct eapol_sm
+#define STATE_MACHINE_DEBUG_PREFIX "EAPOL"
 
 
 /* IEEE 802.1X-2004 - Supplicant - EAPOL state machines */
@@ -150,6 +152,10 @@ struct eapol_sm {
 #define IEEE8021X_KEY_INDEX_FLAG 0x80
 #define IEEE8021X_KEY_INDEX_MASK 0x03
 
+#ifdef _MSC_VER
+#pragma pack(push, 1)
+#endif /* _MSC_VER */
+
 struct ieee802_1x_eapol_key {
 	u8 type;
 	/* Note: key_length is unaligned */
@@ -172,7 +178,11 @@ struct ieee802_1x_eapol_key {
 	 * represents the number of least significant octets from
 	 * MS-MPPE-Send-Key attribute to be used as the keying material;
 	 * RC4 key used in encryption = Key-IV + MS-MPPE-Recv-Key */
-} __attribute__ ((packed));
+} STRUCT_PACKED;
+
+#ifdef _MSC_VER
+#pragma pack(pop)
+#endif /* _MSC_VER */
 
 
 static void eapol_sm_txLogoff(struct eapol_sm *sm);
@@ -185,29 +195,6 @@ static void eapol_sm_abort_cached(struct eapol_sm *sm);
 static void eapol_sm_step_timeout(void *eloop_ctx, void *timeout_ctx);
 
 
-/* Definitions for clarifying state machine implementation */
-#define SM_STATE(machine, state) \
-static void sm_ ## machine ## _ ## state ## _Enter(struct eapol_sm *sm, \
-	int global)
-
-#define SM_ENTRY(machine, state) \
-if (!global || sm->machine ## _state != machine ## _ ## state) { \
-	sm->changed = TRUE; \
-	wpa_printf(MSG_DEBUG, "EAPOL: " #machine " entering state " #state); \
-} \
-sm->machine ## _state = machine ## _ ## state;
-
-#define SM_ENTER(machine, state) \
-sm_ ## machine ## _ ## state ## _Enter(sm, 0)
-#define SM_ENTER_GLOBAL(machine, state) \
-sm_ ## machine ## _ ## state ## _Enter(sm, 1)
-
-#define SM_STEP(machine) \
-static void sm_ ## machine ## _Step(struct eapol_sm *sm)
-
-#define SM_STEP_RUN(machine) sm_ ## machine ## _Step(sm)
-
-
 /* Port Timers state machine - implemented as a function that will be called
  * once a second as a registered event loop timeout */
 static void eapol_port_timers_tick(void *eloop_ctx, void *timeout_ctx)
@@ -1033,7 +1020,8 @@ int eapol_sm_get_status(struct eapol_sm *sm, char *buf, size_t buflen,
  */
 int eapol_sm_get_mib(struct eapol_sm *sm, char *buf, size_t buflen)
 {
-	int len;
+	size_t len;
+
 	if (sm == NULL)
 		return 0;
 	len = snprintf(buf, buflen,
@@ -1043,7 +1031,20 @@ int eapol_sm_get_mib(struct eapol_sm *sm, char *buf, size_t buflen)
 		       "dot1xSuppStartPeriod=%u\n"
 		       "dot1xSuppMaxStart=%u\n"
 		       "dot1xSuppSuppControlledPortStatus=%s\n"
-		       "dot1xSuppBackendPaeState=%d\n"
+		       "dot1xSuppBackendPaeState=%d\n",
+		       sm->SUPP_PAE_state,
+		       sm->heldPeriod,
+		       sm->authPeriod,
+		       sm->startPeriod,
+		       sm->maxStart,
+		       sm->suppPortStatus == Authorized ?
+		       "Authorized" : "Unauthorized",
+		       sm->SUPP_BE_state);
+
+	if (len >= buflen)
+		return len;
+
+	len += snprintf(buf + len, buflen - len,
 		       "dot1xSuppEapolFramesRx=%u\n"
 		       "dot1xSuppEapolFramesTx=%u\n"
 		       "dot1xSuppEapolStartFramesTx=%u\n"
@@ -1055,14 +1056,6 @@ int eapol_sm_get_mib(struct eapol_sm *sm, char *buf, size_t buflen)
 		       "dot1xSuppEapLengthErrorFramesRx=%u\n"
 		       "dot1xSuppLastEapolFrameVersion=%u\n"
 		       "dot1xSuppLastEapolFrameSource=" MACSTR "\n",
-		       sm->SUPP_PAE_state,
-		       sm->heldPeriod,
-		       sm->authPeriod,
-		       sm->startPeriod,
-		       sm->maxStart,
-		       sm->suppPortStatus == Authorized ?
-		       "Authorized" : "Unauthorized",
-		       sm->SUPP_BE_state,
 		       sm->dot1xSuppEapolFramesRx,
 		       sm->dot1xSuppEapolFramesTx,
 		       sm->dot1xSuppEapolStartFramesTx,
@@ -1074,6 +1067,7 @@ int eapol_sm_get_mib(struct eapol_sm *sm, char *buf, size_t buflen)
 		       sm->dot1xSuppEapLengthErrorFramesRx,
 		       sm->dot1xSuppLastEapolFrameVersion,
 		       MAC2STR(sm->dot1xSuppLastEapolFrameSource));
+
 	return len;
 }
 
@@ -1092,8 +1086,9 @@ int eapol_sm_rx_eapol(struct eapol_sm *sm, const u8 *src, const u8 *buf,
 {
 	const struct ieee802_1x_hdr *hdr;
 	const struct ieee802_1x_eapol_key *key;
-	int plen, data_len;
+	int data_len;
 	int res = 1;
+	size_t plen;
 
 	if (sm == NULL)
 		return 0;
@@ -1398,13 +1393,7 @@ static void eapol_sm_abort_cached(struct eapol_sm *sm)
 	sm->cached_pmk = FALSE;
 	sm->SUPP_PAE_state = SUPP_PAE_CONNECTING;
 	sm->suppPortStatus = Unauthorized;
-
-	/* Make sure we do not start sending EAPOL-Start frames first, but
-	 * instead move to RESTART state to start EAPOL authentication. */
-	sm->startWhen = 3;
-
-	if (sm->ctx->aborted_cached)
-		sm->ctx->aborted_cached(sm->ctx->ctx);
+	sm->eapRestart= TRUE;
 }
 
 
@@ -1646,14 +1635,14 @@ eapol_sm_get_config_blob(void *ctx, const char *name)
 
 static struct eapol_callbacks eapol_cb =
 {
-	.get_config = eapol_sm_get_config,
-	.get_bool = eapol_sm_get_bool,
-	.set_bool = eapol_sm_set_bool,
-	.get_int = eapol_sm_get_int,
-	.set_int = eapol_sm_set_int,
-	.get_eapReqData = eapol_sm_get_eapReqData,
-	.set_config_blob = eapol_sm_set_config_blob,
-	.get_config_blob = eapol_sm_get_config_blob,
+	eapol_sm_get_config,
+	eapol_sm_get_bool,
+	eapol_sm_set_bool,
+	eapol_sm_get_int,
+	eapol_sm_set_int,
+	eapol_sm_get_eapReqData,
+	eapol_sm_set_config_blob,
+	eapol_sm_get_config_blob
 };
 
 
