@@ -15,17 +15,7 @@
  * Not used in production version.
  */
 
-#include <stdio.h>
-#include <stdlib.h>
-#include <stdarg.h>
-#include <unistd.h>
-#include <ctype.h>
-#include <string.h>
-#include <signal.h>
-#ifndef CONFIG_NATIVE_WINDOWS
-#include <netinet/in.h>
-#include <arpa/inet.h>
-#endif /* CONFIG_NATIVE_WINDOWS */
+#include "includes.h"
 #include <assert.h>
 
 #include "common.h"
@@ -46,7 +36,7 @@
 extern int wpa_debug_level;
 extern int wpa_debug_show_keys;
 
-struct wpa_driver_ops *wpa_supplicant_drivers[] = { };
+struct wpa_driver_ops *wpa_supplicant_drivers[] = { NULL };
 
 
 struct eapol_test_data {
@@ -82,7 +72,7 @@ static struct eapol_test_data eapol_test;
 static void send_eap_request_identity(void *eloop_ctx, void *timeout_ctx);
 
 
-void hostapd_logger(void *ctx, const u8 *addr, unsigned int module, int level,
+void hostapd_logger(void *ctx, u8 *addr, unsigned int module, int level,
 		    char *fmt, ...)
 {
 	char *format;
@@ -295,23 +285,27 @@ static int eapol_test_compare_pmk(struct eapol_test_data *e)
 
 	if (eapol_sm_get_key(e->wpa_s->eapol, pmk, PMK_LEN) == 0) {
 		wpa_hexdump(MSG_DEBUG, "PMK from EAPOL", pmk, PMK_LEN);
-		if (memcmp(pmk, e->authenticator_pmk, PMK_LEN) != 0)
+		if (memcmp(pmk, e->authenticator_pmk, PMK_LEN) != 0) {
 			printf("WARNING: PMK mismatch\n");
-		else if (e->radius_access_accept_received)
+			wpa_hexdump(MSG_DEBUG, "PMK from AS",
+				    e->authenticator_pmk, PMK_LEN);
+		} else if (e->radius_access_accept_received)
 			ret = 0;
 	} else if (e->authenticator_pmk_len == 16 &&
 		   eapol_sm_get_key(e->wpa_s->eapol, pmk, 16) == 0) {
 		wpa_hexdump(MSG_DEBUG, "LEAP PMK from EAPOL", pmk, 16);
-		if (memcmp(pmk, e->authenticator_pmk, 16) != 0)
+		if (memcmp(pmk, e->authenticator_pmk, 16) != 0) {
 			printf("WARNING: PMK mismatch\n");
-		else if (e->radius_access_accept_received)
+			wpa_hexdump(MSG_DEBUG, "PMK from AS",
+				    e->authenticator_pmk, 16);
+		} else if (e->radius_access_accept_received)
 			ret = 0;
 	} else if (e->radius_access_accept_received && e->no_mppe_keys) {
 		/* No keying material expected */
 		ret = 0;
 	}
 
-	if (ret)
+	if (ret && !e->no_mppe_keys)
 		e->num_mppe_mismatch++;
 	else if (!e->no_mppe_keys)
 		e->num_mppe_ok++;
@@ -406,7 +400,10 @@ static void test_eapol_clean(struct eapol_test_data *e,
 	free(e->radius_conf);
 	e->radius_conf = NULL;
 	scard_deinit(wpa_s->scard);
-	wpa_supplicant_ctrl_iface_deinit(wpa_s);
+	if (wpa_s->ctrl_iface) {
+		wpa_supplicant_ctrl_iface_deinit(wpa_s->ctrl_iface);
+		wpa_s->ctrl_iface = NULL;
+	}
 	wpa_config_free(wpa_s->conf);
 }
 
@@ -651,6 +648,25 @@ ieee802_1x_receive_auth(struct radius_msg *msg, struct radius_msg *req,
 }
 
 
+static int wpa_supplicant_scard_init(struct wpa_supplicant *wpa_s,
+				     struct wpa_ssid *ssid)
+{
+	if (ssid->pcsc == NULL || wpa_s->scard != NULL)
+		return 0;
+	wpa_printf(MSG_DEBUG, "Selected network is configured to use SIM - "
+		   "initialize PCSC");
+	wpa_s->scard = scard_init(SCARD_TRY_BOTH);
+	if (wpa_s->scard == NULL) {
+		wpa_printf(MSG_WARNING, "Failed to initialize SIM "
+			   "(pcsc-lite)");
+		return -1;
+	}
+	eapol_sm_register_scard_ctx(wpa_s->eapol, wpa_s->scard);
+
+	return 0;
+}
+
+
 static void wpa_init_conf(struct eapol_test_data *e,
 			  struct wpa_supplicant *wpa_s, const char *authsrv,
 			  int port, const char *secret)
@@ -670,7 +686,7 @@ static void wpa_init_conf(struct eapol_test_data *e,
 	as = malloc(sizeof(struct hostapd_radius_server));
 	assert(as != NULL);
 	memset(as, 0, sizeof(*as));
-#ifdef CONFIG_NATIVE_WINDOWS
+#if defined(CONFIG_NATIVE_WINDOWS) || defined(CONFIG_ANSI_C_EXTRA)
 	{
 		int a[4];
 		u8 *pos;
@@ -681,9 +697,9 @@ static void wpa_init_conf(struct eapol_test_data *e,
 		*pos++ = a[2];
 		*pos++ = a[3];
 	}
-#else /* CONFIG_NATIVE_WINDOWS */
+#else /* CONFIG_NATIVE_WINDOWS or CONFIG_ANSI_C_EXTRA */
 	inet_aton(authsrv, &as->addr.u.v4);
-#endif /* CONFIG_NATIVE_WINDOWS */
+#endif /* CONFIG_NATIVE_WINDOWS or CONFIG_ANSI_C_EXTRA */
 	as->addr.af = AF_INET;
 	as->port = port;
 	as->shared_secret = (u8 *) strdup(secret);
@@ -715,7 +731,8 @@ static int scard_test(void)
 	unsigned char rand_[num_triplets][16];
 	unsigned char sres_[num_triplets][4];
 	unsigned char kc_[num_triplets][8];
-	int i, j, res;
+	int i, res;
+	size_t j;
 
 #define AKA_RAND_LEN 16
 #define AKA_AUTN_LEN 16
@@ -815,7 +832,8 @@ static int scard_get_triplets(int argc, char *argv[])
 	unsigned char sres[4];
 	unsigned char kc[8];
 	int num_triplets;
-	int i, j;
+	int i;
+	size_t j;
 
 	if (argc < 2 || ((num_triplets = atoi(argv[1])) <= 0)) {
 		printf("invalid parameters for sim command\n");
@@ -886,8 +904,8 @@ static void usage(void)
 	       "[-s<AS secret>] [-r<count>]\n"
 	       "eapol_test scard\n"
 	       "eapol_test sim <PIN> <num triplets> [debug]\n"
-	       "\n"
-	       "options:\n"
+	       "\n");
+	printf("options:\n"
 	       "  -c<conf> = configuration file\n"
 	       "  -a<AS IP> = IP address of the authentication server, "
 	       "default 127.0.0.1\n"
@@ -911,13 +929,8 @@ int main(int argc, char *argv[])
 	char *as_secret = "radius";
 	char *conf = NULL;
 
-#ifdef CONFIG_NATIVE_WINDOWS
-	WSADATA wsaData;
-	if (WSAStartup(MAKEWORD(2, 0), &wsaData)) {
-		printf("Could not find a usable WinSock.dll\n");
+	if (os_program_init())
 		return -1;
-	}
-#endif /* CONFIG_NATIVE_WINDOWS */
 
 	memset(&eapol_test, 0, sizeof(eapol_test));
 
@@ -974,7 +987,10 @@ int main(int argc, char *argv[])
 		return -1;
 	}
 
-	eloop_init(&wpa_s);
+	if (eloop_init(&wpa_s)) {
+		wpa_printf(MSG_ERROR, "Failed to initialize event loop");
+		return -1;
+	}
 
 	memset(&wpa_s, 0, sizeof(wpa_s));
 	eapol_test.wpa_s = &wpa_s;
@@ -989,7 +1005,8 @@ int main(int argc, char *argv[])
 	}
 
 	wpa_init_conf(&eapol_test, &wpa_s, as_addr, as_port, as_secret);
-	if (wpa_supplicant_ctrl_iface_init(&wpa_s)) {
+	wpa_s.ctrl_iface = wpa_supplicant_ctrl_iface_init(&wpa_s);
+	if (wpa_s.ctrl_iface == NULL) {
 		printf("Failed to initialize control interface '%s'.\n"
 		       "You may have another eapol_test process already "
 		       "running or the file was\n"
@@ -1007,18 +1024,16 @@ int main(int argc, char *argv[])
 		return -1;
 
 	if (wait_for_monitor)
-		wpa_supplicant_ctrl_iface_wait(&wpa_s);
+		wpa_supplicant_ctrl_iface_wait(wpa_s.ctrl_iface);
 
 	eloop_register_timeout(30, 0, eapol_test_timeout, &eapol_test, NULL);
 	eloop_register_timeout(0, 0, send_eap_request_identity, &wpa_s, NULL);
-	eloop_register_signal(SIGINT, eapol_test_terminate, NULL);
-	eloop_register_signal(SIGTERM, eapol_test_terminate, NULL);
-#ifndef CONFIG_NATIVE_WINDOWS
-	eloop_register_signal(SIGHUP, eapol_test_terminate, NULL);
-#endif /* CONFIG_NATIVE_WINDOWS */
+	eloop_register_signal_terminate(eapol_test_terminate, NULL);
+	eloop_register_signal_reconfig(eapol_test_terminate, NULL);
 	eloop_run();
 
-	if (eapol_test_compare_pmk(&eapol_test) == 0)
+	if (eapol_test_compare_pmk(&eapol_test) == 0 ||
+	    eapol_test.no_mppe_keys)
 		ret = 0;
 	if (eapol_test.auth_timed_out)
 		ret = -2;
@@ -1041,9 +1056,7 @@ int main(int argc, char *argv[])
 	else
 		printf("SUCCESS\n");
 
-#ifdef CONFIG_NATIVE_WINDOWS
-	WSACleanup();
-#endif /* CONFIG_NATIVE_WINDOWS */
+	os_program_deinit();
 
 	return ret;
 }
