@@ -12,11 +12,10 @@
  * See README and COPYING for more details.
  */
 
-#include <stdlib.h>
-#include <stdio.h>
-#include <unistd.h>
-#include <string.h>
+#include "includes.h"
+#ifdef __linux__
 #include <fcntl.h>
+#endif /* __linux__ */
 
 #include "common.h"
 #include "wpa_supplicant_i.h"
@@ -25,7 +24,11 @@
 extern const char *wpa_supplicant_version;
 extern const char *wpa_supplicant_license;
 #ifndef CONFIG_NO_STDOUT_DEBUG
-extern const char *wpa_supplicant_full_license;
+extern const char *wpa_supplicant_full_license1;
+extern const char *wpa_supplicant_full_license2;
+extern const char *wpa_supplicant_full_license3;
+extern const char *wpa_supplicant_full_license4;
+extern const char *wpa_supplicant_full_license5;
 #endif /* CONFIG_NO_STDOUT_DEBUG */
 
 extern struct wpa_driver_ops *wpa_supplicant_drivers[];
@@ -64,8 +67,8 @@ static void usage(void)
 	       "  -K = include keys (passwords, etc.) in debug output\n"
 	       "  -t = include timestamp in debug messages\n"
 	       "  -h = show this help text\n"
-	       "  -L = show license (GPL and BSD)\n"
-	       "  -p = driver parameters\n"
+	       "  -L = show license (GPL and BSD)\n");
+	printf("  -p = driver parameters\n"
 	       "  -P = PID file\n"
 	       "  -q = decrease debugging verbosity (-qq even less)\n"
 	       "  -v = show version\n"
@@ -82,14 +85,20 @@ static void usage(void)
 static void license(void)
 {
 #ifndef CONFIG_NO_STDOUT_DEBUG
-	printf("%s\n\n%s\n",
-	       wpa_supplicant_version, wpa_supplicant_full_license);
+	printf("%s\n\n%s%s%s%s%s\n",
+	       wpa_supplicant_version,
+	       wpa_supplicant_full_license1,
+	       wpa_supplicant_full_license2,
+	       wpa_supplicant_full_license3,
+	       wpa_supplicant_full_license4,
+	       wpa_supplicant_full_license5);
 #endif /* CONFIG_NO_STDOUT_DEBUG */
 }
 
 
 static void wpa_supplicant_fd_workaround(void)
 {
+#ifdef __linux__
 	int s, i;
 	/* When started from pcmcia-cs scripts, wpa_supplicant might start with
 	 * fd 0, 1, and 2 closed. This will cause some issues because many
@@ -103,6 +112,7 @@ static void wpa_supplicant_fd_workaround(void)
 			break;
 		}
 	}
+#endif /* __linux__ */
 }
 
 
@@ -110,17 +120,12 @@ int main(int argc, char *argv[])
 {
 	int c, i;
 	struct wpa_interface *ifaces, *iface;
-	int iface_count, exitcode = -1;
+	int iface_count, exitcode;
 	struct wpa_params params;
 	struct wpa_global *global;
 
-#ifdef CONFIG_NATIVE_WINDOWS
-	WSADATA wsaData;
-	if (WSAStartup(MAKEWORD(2, 0), &wsaData)) {
-		printf("Could not find a usable WinSock.dll\n");
+	if (os_program_init())
 		return -1;
-	}
-#endif /* CONFIG_NATIVE_WINDOWS */
 
 	memset(&params, 0, sizeof(params));
 	params.wpa_debug_level = MSG_INFO;
@@ -155,7 +160,7 @@ int main(int argc, char *argv[])
 			printf("Debugging disabled with "
 			       "CONFIG_NO_STDOUT_DEBUG=y build time "
 			       "option.\n");
-			goto out;
+			return -1;
 #else /* CONFIG_NO_STDOUT_DEBUG */
 			params.wpa_debug_level--;
 			break;
@@ -165,7 +170,7 @@ int main(int argc, char *argv[])
 			break;
 		case 'h':
 			usage();
-			goto out;
+			return -1;
 		case 'i':
 			iface->ifname = optarg;
 			break;
@@ -174,12 +179,12 @@ int main(int argc, char *argv[])
 			break;
 		case 'L':
 			license();
-			goto out;
+			return -1;
 		case 'p':
 			iface->driver_param = optarg;
 			break;
 		case 'P':
-			params.pid_file = rel2abs_path(optarg);
+			params.pid_file = os_rel2abs_path(optarg);
 			break;
 		case 'q':
 			params.wpa_debug_level++;
@@ -189,7 +194,7 @@ int main(int argc, char *argv[])
 			break;
 		case 'v':
 			printf("%s\n", wpa_supplicant_version);
-			goto out;
+			return -1;
 		case 'w':
 			params.wait_for_interface++;
 			break;
@@ -200,15 +205,17 @@ int main(int argc, char *argv[])
 			iface_count++;
 			iface = realloc(ifaces, iface_count *
 					sizeof(struct wpa_interface));
-			if (iface == NULL)
-				goto out;
+			if (iface == NULL) {
+				free(ifaces);
+				return -1;
+			}
 			ifaces = iface;
 			iface = &ifaces[iface_count - 1]; 
 			memset(iface, 0, sizeof(*iface));
 			break;
 		default:
 			usage();
-			goto out;
+			return -1;
 		}
 	}
 
@@ -226,8 +233,7 @@ int main(int argc, char *argv[])
 			if (iface_count == 1 && params.ctrl_interface)
 				break;
 			usage();
-			exitcode = -1;
-			break;
+			return -1;
 		}
 		if (wpa_supplicant_add_iface(global, &ifaces[i]) == NULL)
 			exitcode = -1;
@@ -238,13 +244,10 @@ int main(int argc, char *argv[])
 
 	wpa_supplicant_deinit(global);
 
-out:
 	free(ifaces);
 	free(params.pid_file);
 
-#ifdef CONFIG_NATIVE_WINDOWS
-	WSACleanup();
-#endif /* CONFIG_NATIVE_WINDOWS */
+	os_program_deinit();
 
 	return exitcode;
 }
