@@ -12,9 +12,7 @@
  * See README and COPYING for more details.
  */
 
-#include <stdlib.h>
-#include <stdio.h>
-#include <string.h>
+#include "includes.h"
 #include <winscard.h>
 
 #include "common.h"
@@ -83,19 +81,145 @@
 typedef enum { SCARD_GSM_SIM, SCARD_USIM } sim_types;
 
 struct scard_data {
-	long ctx;
-	long card;
-	unsigned long protocol;
+	SCARDCONTEXT ctx;
+	SCARDHANDLE card;
+	DWORD protocol;
 	sim_types sim_type;
+	int pin1_required;
 };
 
+#ifdef __MINGW32_VERSION
+/* MinGW does not yet support WinScard, so load the needed functions
+ * dynamically from winscard.dll for now. */
+
+static HINSTANCE dll = NULL; /* winscard.dll */
+
+static const SCARD_IO_REQUEST *dll_g_rgSCardT0Pci, *dll_g_rgSCardT1Pci;
+#undef SCARD_PCI_T0
+#define SCARD_PCI_T0 (dll_g_rgSCardT0Pci)
+#undef SCARD_PCI_T1
+#define SCARD_PCI_T1 (dll_g_rgSCardT1Pci)
+
+
+static WINSCARDAPI LONG WINAPI
+(*dll_SCardEstablishContext)(IN DWORD dwScope,
+			     IN LPCVOID pvReserved1,
+			     IN LPCVOID pvReserved2,
+			     OUT LPSCARDCONTEXT phContext);
+#define SCardEstablishContext dll_SCardEstablishContext
+
+static long (*dll_SCardReleaseContext)(long hContext);
+#define SCardReleaseContext dll_SCardReleaseContext
+
+static WINSCARDAPI LONG WINAPI
+(*dll_SCardListReadersA)(IN SCARDCONTEXT hContext,
+			 IN LPCSTR mszGroups,
+			 OUT LPSTR mszReaders,
+			 IN OUT LPDWORD pcchReaders);
+#undef SCardListReaders
+#define SCardListReaders dll_SCardListReadersA
+
+static WINSCARDAPI LONG WINAPI
+(*dll_SCardConnectA)(IN SCARDCONTEXT hContext,
+		     IN LPCSTR szReader,
+		     IN DWORD dwShareMode,
+		     IN DWORD dwPreferredProtocols,
+		     OUT LPSCARDHANDLE phCard,
+		     OUT LPDWORD pdwActiveProtocol);
+#undef SCardConnect
+#define SCardConnect dll_SCardConnectA
+
+static WINSCARDAPI LONG WINAPI
+(*dll_SCardDisconnect)(IN SCARDHANDLE hCard,
+		       IN DWORD dwDisposition);
+#define SCardDisconnect dll_SCardDisconnect
+
+static WINSCARDAPI LONG WINAPI
+(*dll_SCardTransmit)(IN SCARDHANDLE hCard,
+		     IN LPCSCARD_IO_REQUEST pioSendPci,
+		     IN LPCBYTE pbSendBuffer,
+		     IN DWORD cbSendLength,
+		     IN OUT LPSCARD_IO_REQUEST pioRecvPci,
+		     OUT LPBYTE pbRecvBuffer,
+		     IN OUT LPDWORD pcbRecvLength);
+#define SCardTransmit dll_SCardTransmit
+
+static WINSCARDAPI LONG WINAPI
+(*dll_SCardBeginTransaction)(IN SCARDHANDLE hCard);
+#define SCardBeginTransaction dll_SCardBeginTransaction
+
+static WINSCARDAPI LONG WINAPI
+(*dll_SCardEndTransaction)(IN SCARDHANDLE hCard, IN DWORD dwDisposition);
+#define SCardEndTransaction dll_SCardEndTransaction
+
+
+static int mingw_load_symbols(void)
+{
+	char *sym;
+
+	if (dll)
+		return 0;
+
+	dll = LoadLibrary("winscard");
+	if (dll == NULL) {
+		wpa_printf(MSG_DEBUG, "WinSCard: Could not load winscard.dll "
+			   "library");
+		return -1;
+	}
+
+#define LOADSYM(s) \
+	sym = #s; \
+	dll_ ## s = (void *) GetProcAddress(dll, sym); \
+	if (dll_ ## s == NULL) \
+		goto fail;
+
+	LOADSYM(SCardEstablishContext);
+	LOADSYM(SCardReleaseContext);
+	LOADSYM(SCardListReadersA);
+	LOADSYM(SCardConnectA);
+	LOADSYM(SCardDisconnect);
+	LOADSYM(SCardTransmit);
+	LOADSYM(SCardBeginTransaction);
+	LOADSYM(SCardEndTransaction);
+	LOADSYM(g_rgSCardT0Pci);
+	LOADSYM(g_rgSCardT1Pci);
+
+#undef LOADSYM
+
+	return 0;
+
+fail:
+	wpa_printf(MSG_DEBUG, "WinSCard: Could not get address for %s from "
+		   "winscard.dll", sym);
+	FreeLibrary(dll);
+	dll = NULL;
+	return -1;
+}
+
+
+static void mingw_unload_symbols(void)
+{
+	if (dll == NULL)
+		return;
+
+	FreeLibrary(dll);
+	dll = NULL;
+}
+
+#else /* __MINGW32_VERSION */
+
+#define mingw_load_symbols() 0
+#define mingw_unload_symbols() do { } while (0)
+
+#endif /* __MINGW32_VERSION */
+
 
 static int _scard_select_file(struct scard_data *scard, unsigned short file_id,
 			      unsigned char *buf, size_t *buf_len,
 			      sim_types sim_type, unsigned char *aid);
 static int scard_select_file(struct scard_data *scard, unsigned short file_id,
 			     unsigned char *buf, size_t *buf_len);
-static int scard_verify_pin(struct scard_data *scard, char *pin);
+static int scard_verify_pin(struct scard_data *scard, const char *pin);
 
 
 static int scard_parse_fsp_templ(unsigned char *buf, size_t buf_len,
@@ -182,15 +306,19 @@ static int scard_pin_needed(struct scard_data *scard,
 }
 
 
-struct scard_data * scard_init(scard_sim_type sim_type, char *pin)
+struct scard_data * scard_init(scard_sim_type sim_type)
 {
-	long ret, len;
+	long ret;
+	unsigned long len;
 	struct scard_data *scard;
 	char *readers = NULL;
-	char buf[100];
+	unsigned char buf[100];
 	size_t blen;
+	int transaction = 0;
 
 	wpa_printf(MSG_DEBUG, "SCARD: initializing smart card interface");
+	if (mingw_load_symbols())
+		return NULL;
 	scard = malloc(sizeof(*scard));
 	if (scard == NULL)
 		return NULL;
@@ -237,7 +365,7 @@ struct scard_data * scard_init(scard_sim_type sim_type, char *pin)
 	ret = SCardConnect(scard->ctx, readers, SCARD_SHARE_SHARED,
 			   SCARD_PROTOCOL_T0, &scard->card, &scard->protocol);
 	if (ret != SCARD_S_SUCCESS) {
-		if (ret == SCARD_E_NO_SMARTCARD)
+		if (ret == (long) SCARD_E_NO_SMARTCARD)
 			wpa_printf(MSG_INFO, "No smart card inserted.");
 		else
 			wpa_printf(MSG_WARNING, "SCardConnect err=%lx", ret);
@@ -247,8 +375,17 @@ struct scard_data * scard_init(scard_sim_type sim_type, char *pin)
 	free(readers);
 	readers = NULL;
 
-	wpa_printf(MSG_DEBUG, "SCARD: card=%ld active_protocol=%lu",
-		   scard->card, scard->protocol);
+	wpa_printf(MSG_DEBUG, "SCARD: card=0x%x active_protocol=%lu (%s)",
+		   (unsigned int) scard->card, scard->protocol,
+		   scard->protocol == SCARD_PROTOCOL_T0 ? "T0" : "T1");
+
+	ret = SCardBeginTransaction(scard->card);
+	if (ret != SCARD_S_SUCCESS) {
+		wpa_printf(MSG_DEBUG, "SCARD: Could not begin transaction: "
+			   "0x%x", (unsigned int) ret);
+		goto failed;
+	}
+	transaction = 1;
 
 	blen = sizeof(buf);
 
@@ -284,6 +421,7 @@ struct scard_data * scard_init(scard_sim_type sim_type, char *pin)
 		/* Select based on AID = 3G RID */
 		blen = sizeof(buf);
 		if (_scard_select_file(scard, 0, buf, &blen, scard->sim_type,
+				       (unsigned char *)
 				       "\xA0\x00\x00\x00\x87")) {
 			wpa_printf(MSG_DEBUG, "SCARD: Failed to read 3G RID "
 				   "AID");
@@ -293,31 +431,50 @@ struct scard_data * scard_init(scard_sim_type sim_type, char *pin)
 
 	/* Verify whether CHV1 (PIN1) is needed to access the card. */
 	if (scard_pin_needed(scard, buf, blen)) {
+		scard->pin1_required = 1;
 		wpa_printf(MSG_DEBUG, "PIN1 needed for SIM access");
-		if (pin == NULL) {
-			wpa_printf(MSG_INFO, "No PIN configured for SIM "
-				   "access");
-			/* TODO: ask PIN from user through a frontend (e.g.,
-			 * wpa_cli) */
-			goto failed;
-		}
-		if (scard_verify_pin(scard, pin)) {
-			wpa_printf(MSG_INFO, "PIN verification failed for "
-				"SIM access");
-			/* TODO: what to do? */
-			goto failed;
-		}
+	}
+
+	ret = SCardEndTransaction(scard->card, SCARD_LEAVE_CARD);
+	if (ret != SCARD_S_SUCCESS) {
+		wpa_printf(MSG_DEBUG, "SCARD: Could not end transaction: "
+			   "0x%x", (unsigned int) ret);
 	}
 
 	return scard;
 
 failed:
+	if (transaction)
+		SCardEndTransaction(scard->card, SCARD_LEAVE_CARD);
 	free(readers);
 	scard_deinit(scard);
 	return NULL;
 }
 
 
+int scard_set_pin(struct scard_data *scard, const char *pin)
+{
+	if (scard == NULL)
+		return -1;
+
+	/* Verify whether CHV1 (PIN1) is needed to access the card. */
+	if (scard->pin1_required) {
+		if (pin == NULL) {
+			wpa_printf(MSG_DEBUG, "No PIN configured for SIM "
+				   "access");
+			return -1;
+		}
+		if (scard_verify_pin(scard, pin)) {
+			wpa_printf(MSG_INFO, "PIN verification failed for "
+				"SIM access");
+			return -1;
+		}
+	}
+
+	return 0;
+}
+
+
 void scard_deinit(struct scard_data *scard)
 {
 	long ret;
@@ -342,6 +499,7 @@ void scard_deinit(struct scard_data *scard)
 		}
 	}
 	free(scard);
+	mingw_unload_symbols();
 }
 
 
@@ -459,11 +617,13 @@ static int scard_select_file(struct scard_data *scard, unsigned short file_id,
 static int scard_read_file(struct scard_data *scard,
 			   unsigned char *data, size_t len)
 {
-	char cmd[5] = { SIM_CMD_READ_BIN, len };
+	unsigned char cmd[5] = { SIM_CMD_READ_BIN /* , len */ };
 	size_t blen = len + 3;
 	unsigned char *buf;
 	long ret;
 
+	cmd[4] = len;
+
 	buf = malloc(blen);
 	if (buf == NULL)
 		return -1;
@@ -497,11 +657,11 @@ static int scard_read_file(struct scard_data *scard,
 }
 
 
-static int scard_verify_pin(struct scard_data *scard, char *pin)
+static int scard_verify_pin(struct scard_data *scard, const char *pin)
 {
 	long ret;
 	unsigned char resp[3];
-	char cmd[5 + 8] = { SIM_CMD_VERIFY_CHV1 };
+	unsigned char cmd[5 + 8] = { SIM_CMD_VERIFY_CHV1 };
 	size_t len;
 
 	wpa_printf(MSG_DEBUG, "SCARD: verifying PIN");
@@ -531,10 +691,9 @@ static int scard_verify_pin(struct scard_data *scard, char *pin)
 
 int scard_get_imsi(struct scard_data *scard, char *imsi, size_t *len)
 {
-	char buf[100];
-	size_t blen, imsilen;
+	unsigned char buf[100];
+	size_t blen, imsilen, i;
 	char *pos;
-	int i;
 
 	wpa_printf(MSG_DEBUG, "SCARD: reading IMSI from (GSM) EF-IMSI");
 	blen = sizeof(buf);
@@ -706,7 +865,7 @@ int scard_umts_auth(struct scard_data *scard, unsigned char *rand,
 	if (ret != SCARD_S_SUCCESS)
 		return -1;
 
-	if (len >= 0 && len <= sizeof(resp))
+	if (len <= sizeof(resp))
 		wpa_hexdump(MSG_DEBUG, "SCARD: UMTS alg response", resp, len);
 
 	if (len == 2 && resp[0] == 0x98 && resp[1] == 0x62) {
@@ -723,7 +882,7 @@ int scard_umts_auth(struct scard_data *scard, unsigned char *rand,
 
 	len = sizeof(buf);
 	ret = scard_transmit(scard, get_resp, sizeof(get_resp), buf, &len);
-	if (ret != SCARD_S_SUCCESS || len < 0 || len > sizeof(buf))
+	if (ret != SCARD_S_SUCCESS || len > sizeof(buf))
 		return -1;
 
 	wpa_hexdump(MSG_DEBUG, "SCARD: UMTS get response result", buf, len);
