@@ -1,6 +1,6 @@
 /*
  * WPA Supplicant - WPA state machine and EAPOL-Key processing
- * Copyright (c) 2003-2005, Jouni Malinen <jkmaline@cc.hut.fi>
+ * Copyright (c) 2003-2006, Jouni Malinen <jkmaline@cc.hut.fi>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 2 as
@@ -12,12 +12,7 @@
  * See README and COPYING for more details.
  */
 
-#include <stdlib.h>
-#include <stdio.h>
-#ifndef CONFIG_NATIVE_WINDOWS
-#include <netinet/in.h>
-#endif /* CONFIG_NATIVE_WINDOWS */
-#include <string.h>
+#include "includes.h"
 
 #include "common.h"
 #include "md5.h"
@@ -31,6 +26,7 @@
 #include "l2_packet.h"
 #include "eapol_sm.h"
 #include "preauth.h"
+#include "pmksa_cache.h"
 #include "wpa_i.h"
 
 
@@ -43,7 +39,9 @@ static const u8 WPA_AUTH_KEY_MGMT_PSK_OVER_802_1X[] = { 0x00, 0x50, 0xf2, 2 };
 static const u8 WPA_CIPHER_SUITE_NONE[] = { 0x00, 0x50, 0xf2, 0 };
 static const u8 WPA_CIPHER_SUITE_WEP40[] = { 0x00, 0x50, 0xf2, 1 };
 static const u8 WPA_CIPHER_SUITE_TKIP[] = { 0x00, 0x50, 0xf2, 2 };
+#if 0
 static const u8 WPA_CIPHER_SUITE_WRAP[] = { 0x00, 0x50, 0xf2, 3 };
+#endif
 static const u8 WPA_CIPHER_SUITE_CCMP[] = { 0x00, 0x50, 0xf2, 4 };
 static const u8 WPA_CIPHER_SUITE_WEP104[] = { 0x00, 0x50, 0xf2, 5 };
 
@@ -61,13 +59,21 @@ static const u8 WPA_CIPHER_SUITE_WEP104[] = { 0x00, 0x50, 0xf2, 5 };
  * WPA Capabilities (2 octets, little endian) (default: 0)
  */
 
+#ifdef _MSC_VER
+#pragma pack(push, 1)
+#endif /* _MSC_VER */
+
 struct wpa_ie_hdr {
 	u8 elem_id;
 	u8 len;
 	u8 oui[3];
 	u8 oui_type;
 	u8 version[2];
-} __attribute__ ((packed));
+} STRUCT_PACKED;
+
+#ifdef _MSC_VER
+#pragma pack(pop)
+#endif /* _MSC_VER */
 
 
 static const int RSN_SELECTOR_LEN = 4;
@@ -77,7 +83,9 @@ static const u8 RSN_AUTH_KEY_MGMT_PSK_OVER_802_1X[] = { 0x00, 0x0f, 0xac, 2 };
 static const u8 RSN_CIPHER_SUITE_NONE[] = { 0x00, 0x0f, 0xac, 0 };
 static const u8 RSN_CIPHER_SUITE_WEP40[] = { 0x00, 0x0f, 0xac, 1 };
 static const u8 RSN_CIPHER_SUITE_TKIP[] = { 0x00, 0x0f, 0xac, 2 };
+#if 0
 static const u8 RSN_CIPHER_SUITE_WRAP[] = { 0x00, 0x0f, 0xac, 3 };
+#endif
 static const u8 RSN_CIPHER_SUITE_CCMP[] = { 0x00, 0x0f, 0xac, 4 };
 static const u8 RSN_CIPHER_SUITE_WEP104[] = { 0x00, 0x0f, 0xac, 5 };
 
@@ -112,11 +120,15 @@ static const u8 RSN_KEY_DATA_PMKID[] = { 0x00, 0x0f, 0xac, 4 };
  * PMKID List (16 * n octets)
  */
 
+#ifdef _MSC_VER
+#pragma pack(push, 1)
+#endif /* _MSC_VER */
+
 struct rsn_ie_hdr {
 	u8 elem_id; /* WLAN_EID_RSN */
 	u8 len;
 	u8 version[2];
-} __attribute__ ((packed));
+} STRUCT_PACKED;
 
 
 struct wpa_eapol_key {
@@ -132,9 +144,13 @@ struct wpa_eapol_key {
 	u8 key_mic[16];
 	u8 key_data_length[2];
 	/* followed by key_data_length bytes of key_data */
-} __attribute__ ((packed));
+} STRUCT_PACKED;
 
-#define WPA_KEY_INFO_TYPE_MASK (BIT(0) | BIT(1) | BIT(2))
+#ifdef _MSC_VER
+#pragma pack(pop)
+#endif /* _MSC_VER */
+
+#define WPA_KEY_INFO_TYPE_MASK ((u16) (BIT(0) | BIT(1) | BIT(2)))
 #define WPA_KEY_INFO_TYPE_HMAC_MD5_RC4 BIT(0)
 #define WPA_KEY_INFO_TYPE_HMAC_SHA1_AES BIT(1)
 #define WPA_KEY_INFO_KEY_TYPE BIT(3) /* 1 = Pairwise, 0 = Group key */
@@ -288,7 +304,7 @@ static int wpa_parse_wpa_ie_wpa(const u8 *wpa_ie, size_t wpa_ie_len,
 
 	if (hdr->elem_id != GENERIC_INFO_ELEM ||
 	    hdr->len != wpa_ie_len - 2 ||
-	    memcmp(&hdr->oui, WPA_OUI_TYPE, WPA_SELECTOR_LEN) != 0 ||
+	    memcmp(hdr->oui, WPA_OUI_TYPE, WPA_SELECTOR_LEN) != 0 ||
 	    WPA_GET_LE16(hdr->version) != WPA_VERSION) {
 		wpa_printf(MSG_DEBUG, "%s: malformed ie or unknown version",
 			   __func__);
@@ -357,8 +373,9 @@ static int wpa_parse_wpa_ie_wpa(const u8 *wpa_ie, size_t wpa_ie_len,
 	}
 
 	if (left > 0) {
-		wpa_printf(MSG_DEBUG, "%s: ie has %u trailing bytes - ignored",
+		wpa_printf(MSG_DEBUG, "%s: ie has %u trailing bytes",
 			   __func__, left);
+		return -1;
 	}
 
 	return 0;
@@ -520,7 +537,7 @@ static int wpa_gen_wpa_ie_wpa(u8 *wpa_ie, size_t wpa_ie_len,
 
 	hdr = (struct wpa_ie_hdr *) wpa_ie;
 	hdr->elem_id = GENERIC_INFO_ELEM;
-	memcpy(&hdr->oui, WPA_OUI_TYPE, WPA_SELECTOR_LEN);
+	memcpy(hdr->oui, WPA_OUI_TYPE, WPA_SELECTOR_LEN);
 	WPA_PUT_LE16(hdr->version, WPA_VERSION);
 	pos = (u8 *) (hdr + 1);
 
@@ -834,6 +851,83 @@ void wpa_sm_key_request(struct wpa_sm *sm, int error, int pairwise)
 }
 
 
+/**
+ * wpa_sm_stakey_request - Send EAPOL-Key Request for STAKey handshake
+ * @sm: Pointer to WPA state machine data from wpa_sm_init()
+ * @peer: MAC address of the peer STA
+ * Returns: 0 on success, or -1 on failure
+ *
+ * Send an EAPOL-Key Request to the current authenticator to start STAKey
+ * handeshake with the peer.
+ */
+int wpa_sm_stakey_request(struct wpa_sm *sm, const u8 *peer)
+{
+#ifdef CONFIG_STAKEY
+	size_t rlen, kde_len;
+	struct wpa_eapol_key *reply;
+	int key_info, ver, alg;
+	u8 bssid[ETH_ALEN], *rbuf, *pos;
+	struct wpa_ssid *ssid = sm->cur_ssid;
+
+	if (sm->proto != WPA_PROTO_RSN || !sm->ptk_set ||
+	    ssid == NULL || !ssid->stakey)
+		return -1;
+
+	/* TODO: how to select which cipher to use? */
+	alg = sm->pairwise_cipher;
+
+	if (alg == WPA_CIPHER_CCMP)
+		ver = WPA_KEY_INFO_TYPE_HMAC_SHA1_AES;
+	else
+		ver = WPA_KEY_INFO_TYPE_HMAC_MD5_RC4;
+
+	if (wpa_sm_get_bssid(sm, bssid) < 0) {
+		wpa_printf(MSG_WARNING, "Failed to read BSSID for EAPOL-Key "
+			   "STAKey Request");
+		return -1;
+	}
+
+	kde_len = 2 + sizeof(RSN_KEY_DATA_MAC_ADDR) + ETH_ALEN;
+
+	rbuf = wpa_sm_alloc_eapol(sm, IEEE802_1X_TYPE_EAPOL_KEY, NULL,
+				  sizeof(*reply) + kde_len, &rlen,
+				  (void *) &reply);
+	if (rbuf == NULL)
+		return -1;
+
+	reply->type = EAPOL_KEY_TYPE_RSN;
+	key_info = WPA_KEY_INFO_REQUEST | WPA_KEY_INFO_SECURE |
+		WPA_KEY_INFO_MIC | ver;
+	WPA_PUT_BE16(reply->key_info, key_info);
+	WPA_PUT_BE16(reply->key_length, 0);
+	memcpy(reply->replay_counter, sm->request_counter,
+	       WPA_REPLAY_COUNTER_LEN);
+	inc_byte_array(sm->request_counter, WPA_REPLAY_COUNTER_LEN);
+
+	/* MAC address KDE */
+	WPA_PUT_BE16(reply->key_data_length, (u16) kde_len);
+	pos = (u8 *) (reply + 1);
+	*pos++ = GENERIC_INFO_ELEM;
+	*pos++ = kde_len - 2;
+	memcpy(pos, RSN_KEY_DATA_MAC_ADDR, sizeof(RSN_KEY_DATA_MAC_ADDR));
+	pos += sizeof(RSN_KEY_DATA_MAC_ADDR);
+	memcpy(pos, peer, ETH_ALEN);
+
+	wpa_printf(MSG_INFO, "RSN: Sending EAPOL-Key STAKey Request (peer "
+		   MACSTR ")", MAC2STR(peer));
+	wpa_eapol_key_send(sm, sm->ptk.kck, ver, bssid, ETH_P_EAPOL,
+			   rbuf, rlen, reply->key_mic);
+
+	return 0;
+
+#else /* CONFIG_STAKEY */
+
+	return -1;
+
+#endif /* CONFIG_STAKEY */
+}
+
+
 struct wpa_eapol_ie_parse {
 	const u8 *wpa_ie;
 	size_t wpa_ie_len;
@@ -842,6 +936,10 @@ struct wpa_eapol_ie_parse {
 	const u8 *pmkid;
 	const u8 *gtk;
 	size_t gtk_len;
+	const u8 *stakey;
+	size_t stakey_len;
+	const u8 *mac_addr;
+	size_t mac_addr_len;
 };
 
 
@@ -878,6 +976,21 @@ static int wpa_supplicant_parse_generic(const u8 *pos, const u8 *end,
 	    memcmp(pos + 2, RSN_KEY_DATA_GROUPKEY, RSN_SELECTOR_LEN) == 0) {
 		ie->gtk = pos + 2 + RSN_SELECTOR_LEN;
 		ie->gtk_len = pos[1] - RSN_SELECTOR_LEN;
+		return 0;
+	}
+
+	if (pos[1] > RSN_SELECTOR_LEN + 2 &&
+	    memcmp(pos + 2, RSN_KEY_DATA_STAKEY, RSN_SELECTOR_LEN) == 0) {
+		ie->stakey = pos + 2 + RSN_SELECTOR_LEN;
+		ie->stakey_len = pos[1] - RSN_SELECTOR_LEN;
+		return 0;
+	}
+
+	if (pos[1] > RSN_SELECTOR_LEN + 2 &&
+	    memcmp(pos + 2, RSN_KEY_DATA_MAC_ADDR, RSN_SELECTOR_LEN) == 0) {
+		ie->mac_addr = pos + 2 + RSN_SELECTOR_LEN;
+		ie->mac_addr_len = pos[1] - RSN_SELECTOR_LEN;
+		return 0;
 	}
 
 	return 0;
@@ -899,6 +1012,11 @@ static int wpa_supplicant_parse_ies(const u8 *buf, size_t len,
 
 	memset(ie, 0, sizeof(*ie));
 	for (pos = buf, end = pos + len; pos + 1 < end; pos += 2 + pos[1]) {
+		if (pos[0] == 0xdd &&
+		    ((pos == buf + len - 1) || pos[1] == 0)) {
+			/* Ignore padding */
+			break;
+		}
 		if (pos + 2 + pos[1] > end) {
 			wpa_printf(MSG_DEBUG, "WPA: EAPOL-Key Key Data "
 				   "underflow (ie=%d len=%d)", pos[0], pos[1]);
@@ -937,7 +1055,7 @@ static int wpa_supplicant_get_pmk(struct wpa_sm *sm,
 		 * not have enough time to get the association information
 		 * event before receiving this 1/4 message, so try to find a
 		 * matching PMKSA cache entry here. */
-		sm->cur_pmksa = pmksa_cache_get(sm, src_addr, pmkid);
+		sm->cur_pmksa = pmksa_cache_get(sm->pmksa, src_addr, pmkid);
 		if (sm->cur_pmksa) {
 			wpa_printf(MSG_DEBUG, "RSN: found matching PMKID from "
 				   "PMKSA cache");
@@ -968,10 +1086,10 @@ static int wpa_supplicant_get_pmk(struct wpa_sm *sm,
 			wpa_hexdump_key(MSG_DEBUG, "WPA: PMK from EAPOL state "
 					"machines", sm->pmk, pmk_len);
 			sm->pmk_len = pmk_len;
-			pmksa_cache_add(sm, sm->pmk, pmk_len, src_addr,
+			pmksa_cache_add(sm->pmksa, sm->pmk, pmk_len, src_addr,
 					sm->own_addr, sm->cur_ssid);
 			if (!sm->cur_pmksa && pmkid &&
-			    pmksa_cache_get(sm, src_addr, pmkid)) {
+			    pmksa_cache_get(sm->pmksa, src_addr, pmkid)) {
 				wpa_printf(MSG_DEBUG, "RSN: the new PMK "
 					   "matches with the PMKID");
 				abort_cached = 0;
@@ -1210,7 +1328,7 @@ static int wpa_supplicant_install_ptk(struct wpa_sm *sm,
 	}
 
 	if (wpa_sm_set_key(sm, alg, src_addr, 0, 1, key_rsc, rsclen,
-			   (u8 *) &sm->ptk.tk1, keylen) < 0) {
+			   (u8 *) sm->ptk.tk1, keylen) < 0) {
 		wpa_printf(MSG_WARNING, "WPA: Failed to set PTK to the "
 			   "driver.");
 		return -1;
@@ -1341,7 +1459,7 @@ static int wpa_supplicant_gtk_tx_bit_workaround(const struct wpa_sm *sm,
 static int wpa_supplicant_pairwise_gtk(struct wpa_sm *sm,
 				       const unsigned char *src_addr,
 				       const struct wpa_eapol_key *key,
-				       const u8 *gtk, int gtk_len,
+				       const u8 *gtk, size_t gtk_len,
 				       int key_info)
 {
 	struct wpa_gtk_data gd;
@@ -1441,15 +1559,6 @@ static int wpa_supplicant_validate_ie(struct wpa_sm *sm,
 		}
 	}
 
-	if (ie->wpa_ie == NULL && ie->rsn_ie == NULL &&
-	    (sm->ap_wpa_ie || sm->ap_rsn_ie)) {
-		wpa_report_ie_mismatch(sm, "IE in 3/4 msg does not match "
-				       "with IE in Beacon/ProbeResp (no IE?)",
-				       src_addr, ie->wpa_ie, ie->wpa_ie_len,
-				       ie->rsn_ie, ie->rsn_ie_len);
-		return -1;
-	}
-
 	if ((ie->wpa_ie && sm->ap_wpa_ie &&
 	     (ie->wpa_ie_len != sm->ap_wpa_ie_len ||
 	      memcmp(ie->wpa_ie, sm->ap_wpa_ie, ie->wpa_ie_len) != 0)) ||
@@ -1595,10 +1704,126 @@ static void wpa_supplicant_process_3_of_4(struct wpa_sm *sm,
 }
 
 
+#ifdef CONFIG_STAKEY
+static int wpa_supplicant_send_stakey_2_of_2(struct wpa_sm *sm,
+					     const unsigned char *src_addr,
+					     const struct wpa_eapol_key *key,
+					     int ver, u16 key_info,
+					     const u8 *peer)
+{
+	size_t rlen;
+	struct wpa_eapol_key *reply;
+	u8 *rbuf, *pos;
+	size_t kde_len;
+
+	kde_len = 2 + sizeof(RSN_KEY_DATA_MAC_ADDR) + ETH_ALEN;
+
+	rbuf = wpa_sm_alloc_eapol(sm, IEEE802_1X_TYPE_EAPOL_KEY, NULL,
+				  sizeof(*reply) + kde_len, &rlen,
+				  (void *) &reply);
+	if (rbuf == NULL)
+		return -1;
+
+	reply->type = EAPOL_KEY_TYPE_RSN;
+	key_info &= WPA_KEY_INFO_KEY_INDEX_MASK;
+	key_info |= ver | WPA_KEY_INFO_MIC | WPA_KEY_INFO_SECURE;
+	WPA_PUT_BE16(reply->key_info, key_info);
+	WPA_PUT_BE16(reply->key_length, 0);
+	memcpy(reply->replay_counter, key->replay_counter,
+	       WPA_REPLAY_COUNTER_LEN);
+
+	/* MAC address KDE */
+	WPA_PUT_BE16(reply->key_data_length, (u16) kde_len);
+	pos = (u8 *) (reply + 1);
+	*pos++ = GENERIC_INFO_ELEM;
+	*pos++ = kde_len - 2;
+	memcpy(pos, RSN_KEY_DATA_MAC_ADDR, sizeof(RSN_KEY_DATA_MAC_ADDR));
+	pos += sizeof(RSN_KEY_DATA_MAC_ADDR);
+	memcpy(pos, peer, ETH_ALEN);
+
+	wpa_printf(MSG_DEBUG, "RSN: Sending EAPOL-Key STAKey 2/2 for " MACSTR,
+		   MAC2STR(peer));
+	wpa_eapol_key_send(sm, sm->ptk.kck, ver, src_addr, ETH_P_EAPOL,
+			   rbuf, rlen, reply->key_mic);
+
+	return 0;
+}
+
+
+static int wpa_supplicant_process_stakey_1_of_2(
+	struct wpa_sm *sm, const unsigned char *src_addr,
+	const struct wpa_eapol_key *key, u16 key_info,
+	const u8 *stakey, size_t stakey_len)
+{
+	struct wpa_ssid *ssid = sm->cur_ssid;
+	u16 ver;
+	size_t keylen;
+	int cipher;
+	const u8 *_key;
+	u8 key_buf[32], rsc[6];
+
+	wpa_printf(MSG_DEBUG, "RSN: Received STAKey msg 1/2");
+	if (ssid == NULL || !ssid->stakey || sm->proto != WPA_PROTO_RSN) {
+		wpa_printf(MSG_DEBUG, "RSN: STAKey negotiation not allowed for"
+			   " the current network");
+		return -1;
+	}
+	
+	/* STAKey KDE: Reserved(2 octets) PeerSTA Addr(6 octets) STAKey */
+
+	ver = key_info & WPA_KEY_INFO_TYPE_MASK;
+	if (ver == WPA_KEY_INFO_TYPE_HMAC_MD5_RC4) {
+		cipher = WPA_CIPHER_TKIP;
+		keylen = 32;
+	} else if (ver == WPA_KEY_INFO_TYPE_HMAC_SHA1_AES) {
+		cipher = WPA_CIPHER_CCMP;
+		keylen = 16;
+	} else {
+		wpa_printf(MSG_DEBUG, "RSN: Invalid STAKey key_info version "
+			   "%d", ver);
+		return -1;
+	}
+
+	if (stakey_len < 8 + keylen) {
+		wpa_printf(MSG_DEBUG, "RSN: Too short STAKey KDE");
+		return -1;
+	}
+
+	wpa_printf(MSG_DEBUG, "RSN: STAKey for peer STA " MACSTR,
+		   MAC2STR(stakey + 2));
+	wpa_hexdump_key(MSG_DEBUG, "RSN: STAKey", stakey + 8, keylen);
+
+	if (cipher == WPA_CIPHER_TKIP) {
+		/* Swap Tx/Rx keys for Michael MIC */
+		memcpy(key_buf, stakey + 2, 16);
+		memcpy(key_buf + 16, stakey + 2 + 24, 8);
+		memcpy(key_buf + 24, stakey + 2 + 16, 8);
+		_key = key_buf;
+	} else
+		_key = stakey + 8;
+
+	memset(rsc, 0, 6);
+	if (wpa_sm_set_key(sm, cipher, stakey + 2, 0, 0, rsc, sizeof(rsc),
+			   _key, keylen) < 0) {
+		wpa_printf(MSG_WARNING, "RSN: Failed to set STAKey to "
+			   "the driver.");
+		return -1;
+	}
+
+	wpa_supplicant_send_stakey_2_of_2(sm, src_addr, key, ver, key_info,
+		stakey + 2);
+
+	return 0;
+}
+#endif /* CONFIG_STAKEY */
+
+
 static int wpa_supplicant_process_1_of_2_rsn(struct wpa_sm *sm,
+					     const unsigned char *src_addr,
+					     const struct wpa_eapol_key *key,
 					     const u8 *keydata,
 					     size_t keydatalen,
-					     int key_info,
+					     u16 key_info,
 					     struct wpa_gtk_data *gd)
 {
 	int maxkeylen;
@@ -1610,6 +1835,21 @@ static int wpa_supplicant_process_1_of_2_rsn(struct wpa_sm *sm,
 		wpa_printf(MSG_WARNING, "WPA: GTK IE in unencrypted key data");
 		return -1;
 	}
+	if (ie.stakey) {
+#ifdef CONFIG_STAKEY
+		if (!(key_info & WPA_KEY_INFO_ENCR_KEY_DATA)) {
+			wpa_printf(MSG_WARNING, "WPA: STAKey KDE in "
+				   "unencrypted key data");
+			return -1;
+		}
+
+		wpa_supplicant_process_stakey_1_of_2(sm, src_addr, key,
+						     key_info, ie.stakey,
+						     ie.stakey_len);
+#endif /* CONFIG_STAKEY */
+		/* Abort GTK processing since this was a STAKey message. */
+		return 1;
+	}
 	if (ie.gtk == NULL) {
 		wpa_printf(MSG_INFO, "WPA: No GTK IE in Group Key msg 1/2");
 		return -1;
@@ -1640,22 +1880,29 @@ static int wpa_supplicant_process_1_of_2_rsn(struct wpa_sm *sm,
 static int wpa_supplicant_process_1_of_2_wpa(struct wpa_sm *sm,
 					     const struct wpa_eapol_key *key,
 					     size_t keydatalen, int key_info,
-					     int extra_len, u16 ver,
+					     size_t extra_len, u16 ver,
 					     struct wpa_gtk_data *gd)
 {
-	int maxkeylen;
+	size_t maxkeylen;
 	u8 ek[32];
 
 	gd->gtk_len = WPA_GET_BE16(key->key_length);
 	maxkeylen = keydatalen;
 	if (keydatalen > extra_len) {
 		wpa_printf(MSG_INFO, "WPA: Truncated EAPOL-Key packet:"
-			   " key_data_length=%lu > extra_len=%d",
-			   (unsigned long) keydatalen, extra_len);
+			   " key_data_length=%lu > extra_len=%lu",
+			   (unsigned long) keydatalen,
+			   (unsigned long) extra_len);
 		return -1;
 	}
-	if (ver == WPA_KEY_INFO_TYPE_HMAC_SHA1_AES)
+	if (ver == WPA_KEY_INFO_TYPE_HMAC_SHA1_AES) {
+		if (maxkeylen < 8) {
+			wpa_printf(MSG_INFO, "WPA: Too short maxkeylen (%lu)",
+				   (unsigned long) maxkeylen);
+			return -1;
+		}
 		maxkeylen -= 8;
+	}
 
 	if (wpa_supplicant_check_group_cipher(sm->group_cipher,
 					      gd->gtk_len, maxkeylen,
@@ -1743,32 +1990,40 @@ static void wpa_supplicant_process_1_of_2(struct wpa_sm *sm,
 					  int extra_len, u16 ver)
 {
 	u16 key_info, keydatalen;
-	int rekey;
+	int rekey, ret;
 	struct wpa_gtk_data gd;
 
 	memset(&gd, 0, sizeof(gd));
 
 	rekey = wpa_sm_get_state(sm) == WPA_COMPLETED;
-	wpa_sm_set_state(sm, WPA_GROUP_HANDSHAKE);
-	wpa_printf(MSG_DEBUG, "WPA: RX message 1 of Group Key Handshake from "
-		   MACSTR " (ver=%d)", MAC2STR(src_addr), ver);
+	wpa_printf(MSG_DEBUG, "WPA: RX message 1 of Group Key or STAKey "
+		   "Handshake from " MACSTR " (ver=%d)",
+		   MAC2STR(src_addr), ver);
 
 	key_info = WPA_GET_BE16(key->key_info);
 	keydatalen = WPA_GET_BE16(key->key_data_length);
 
 	if (sm->proto == WPA_PROTO_RSN) {
-		if (wpa_supplicant_process_1_of_2_rsn(sm,
-						      (const u8 *) (key + 1),
-						      keydatalen, key_info,
-						      &gd))
-			return;
+		ret = wpa_supplicant_process_1_of_2_rsn(sm, src_addr, key,
+							(const u8 *) (key + 1),
+							keydatalen, key_info,
+							&gd);
 	} else {
-		if (wpa_supplicant_process_1_of_2_wpa(sm, key, keydatalen,
-						      key_info, extra_len,
-						      ver, &gd))
-			return;
+		ret = wpa_supplicant_process_1_of_2_wpa(sm, key, keydatalen,
+							key_info, extra_len,
+							ver, &gd);
 	}
 
+	if (ret == 1) {
+		/* STAKey handshake - abort Group Key processing */
+		return;
+	}
+
+	wpa_sm_set_state(sm, WPA_GROUP_HANDSHAKE);
+
+	if (ret)
+		return;
+
 	if (wpa_supplicant_install_gtk(sm, &gd, key->key_rsc) ||
 	    wpa_supplicant_send_2_of_2(sm, src_addr, key, ver, key_info))
 		return;
@@ -1887,53 +2142,6 @@ static int wpa_supplicant_decrypt_key_data(struct wpa_sm *sm,
 }
 
 
-/**
- * wpa_sm_aborted_cached - Notify WPA that PMKSA caching was aborted
- * @sm: Pointer to WPA state machine data from wpa_sm_init()
- */
-void wpa_sm_aborted_cached(struct wpa_sm *sm)
-{
-	if (sm && sm->cur_pmksa) {
-		wpa_printf(MSG_DEBUG, "RSN: Cancelling PMKSA caching attempt");
-		sm->cur_pmksa = NULL;
-	}
-}
-
-
-static void wpa_eapol_key_dump(const struct wpa_eapol_key *key)
-{
-#ifndef CONFIG_NO_STDOUT_DEBUG
-	u16 key_info = WPA_GET_BE16(key->key_info);
-
-	wpa_printf(MSG_DEBUG, "  EAPOL-Key type=%d", key->type);
-	wpa_printf(MSG_DEBUG, "  key_info 0x%x (ver=%d keyidx=%d rsvd=%d %s"
-		   "%s%s%s%s%s%s%s)",
-		   key_info, key_info & WPA_KEY_INFO_TYPE_MASK,
-		   (key_info & WPA_KEY_INFO_KEY_INDEX_MASK) >>
-		   WPA_KEY_INFO_KEY_INDEX_SHIFT,
-		   (key_info & (BIT(13) | BIT(14) | BIT(15))) >> 13,
-		   key_info & WPA_KEY_INFO_KEY_TYPE ? "Pairwise" : "Group",
-		   key_info & WPA_KEY_INFO_INSTALL ? " Install" : "",
-		   key_info & WPA_KEY_INFO_ACK ? " Ack" : "",
-		   key_info & WPA_KEY_INFO_MIC ? " MIC" : "",
-		   key_info & WPA_KEY_INFO_SECURE ? " Secure" : "",
-		   key_info & WPA_KEY_INFO_ERROR ? " Error" : "",
-		   key_info & WPA_KEY_INFO_REQUEST ? " Request" : "",
-		   key_info & WPA_KEY_INFO_ENCR_KEY_DATA ? " Encr" : "");
-	wpa_printf(MSG_DEBUG, "  key_length=%u key_data_length=%u",
-		   WPA_GET_BE16(key->key_length),
-		   WPA_GET_BE16(key->key_data_length));
-	wpa_hexdump(MSG_DEBUG, "  replay_counter",
-		    key->replay_counter, WPA_REPLAY_COUNTER_LEN);
-	wpa_hexdump(MSG_DEBUG, "  key_nonce", key->key_nonce, WPA_NONCE_LEN);
-	wpa_hexdump(MSG_DEBUG, "  key_iv", key->key_iv, 16);
-	wpa_hexdump(MSG_DEBUG, "  key_rsc", key->key_rsc, 8);
-	wpa_hexdump(MSG_DEBUG, "  key_id (reserved)", key->key_id, 8);
-	wpa_hexdump(MSG_DEBUG, "  key_mic", key->key_mic, 16);
-#endif /* CONFIG_NO_STDOUT_DEBUG */
-}
-
-
 /**
  * wpa_sm_rx_eapol - Process received WPA EAPOL frames
  * @sm: Pointer to WPA state machine data from wpa_sm_init()
@@ -1975,7 +2183,7 @@ int wpa_sm_rx_eapol(struct wpa_sm *sm, const u8 *src_addr,
 
 	hdr = (struct ieee802_1x_hdr *) tmp;
 	key = (struct wpa_eapol_key *) (hdr + 1);
-	plen = ntohs(hdr->length);
+	plen = be_to_host16(hdr->length);
 	data_len = plen + sizeof(*hdr);
 	wpa_printf(MSG_DEBUG, "IEEE 802.1X RX: version=%d type=%d length=%lu",
 		   hdr->version, hdr->type, (unsigned long) plen);
@@ -1986,6 +2194,12 @@ int wpa_sm_rx_eapol(struct wpa_sm *sm, const u8 *src_addr,
 	if (hdr->type != IEEE802_1X_TYPE_EAPOL_KEY) {
 		wpa_printf(MSG_DEBUG, "WPA: EAPOL frame (type %u) discarded, "
 			"not a Key frame", hdr->type);
+		if (sm->cur_pmksa) {
+			wpa_printf(MSG_DEBUG, "WPA: Cancelling PMKSA caching "
+				   "attempt - attempt full EAP "
+				   "authentication");
+			eapol_sm_notify_pmkid_attempt(sm->eapol, 0);
+		}
 		ret = 0;
 		goto out;
 	}
@@ -1997,6 +2211,7 @@ int wpa_sm_rx_eapol(struct wpa_sm *sm, const u8 *src_addr,
 		goto out;
 	}
 
+	wpa_printf(MSG_DEBUG, "  EAPOL-Key type=%d", key->type);
 	if (key->type != EAPOL_KEY_TYPE_WPA && key->type != EAPOL_KEY_TYPE_RSN)
 	{
 		wpa_printf(MSG_DEBUG, "WPA: EAPOL-Key type (%d) unknown, "
@@ -2004,7 +2219,6 @@ int wpa_sm_rx_eapol(struct wpa_sm *sm, const u8 *src_addr,
 		ret = 0;
 		goto out;
 	}
-	wpa_eapol_key_dump(key);
 
 	eapol_sm_notify_lower_layer_success(sm->eapol);
 	wpa_hexdump(MSG_MSGDUMP, "WPA: RX EAPOL-Key", tmp, len);
@@ -2186,14 +2400,16 @@ static const u8 * wpa_cipher_suite(struct wpa_sm *sm, int cipher)
  */
 int wpa_sm_get_mib(struct wpa_sm *sm, char *buf, size_t buflen)
 {
-	int len, i;
+	int i;
 	char pmkid_txt[PMKID_LEN * 2 + 1];
 	int rsna;
+	size_t len;
 
 	if (sm->cur_pmksa) {
-		char *pos = pmkid_txt;
+		char *pos = pmkid_txt, *end = pos + sizeof(pmkid_txt);
 		for (i = 0; i < PMKID_LEN; i++) {
-			pos += sprintf(pos, "%02x", sm->cur_pmksa->pmkid[i]);
+			pos += snprintf(pos, end - pos, "%02x",
+					sm->cur_pmksa->pmkid[i]);
 		}
 	} else
 		pmkid_txt[0] = '\0';
@@ -2216,7 +2432,19 @@ int wpa_sm_get_mib(struct wpa_sm *sm, char *buf, size_t buflen)
 		       "dot11RSNAConfigPMKLifetime=%d\n"
 		       "dot11RSNAConfigPMKReauthThreshold=%d\n"
 		       "dot11RSNAConfigNumberOfPTKSAReplayCounters=1\n"
-		       "dot11RSNAConfigSATimeout=%d\n"
+		       "dot11RSNAConfigSATimeout=%d\n",
+		       rsna ? "TRUE" : "FALSE",
+		       rsna ? "TRUE" : "FALSE",
+		       RSN_VERSION,
+		       wpa_cipher_bits(sm->group_cipher),
+		       sm->dot11RSNAConfigPMKLifetime,
+		       sm->dot11RSNAConfigPMKReauthThreshold,
+		       sm->dot11RSNAConfigSATimeout);
+
+	if (len >= buflen)
+		return len;
+
+	len += snprintf(buf + len, buflen - len,
 		       "dot11RSNAAuthenticationSuiteSelected=" RSN_SUITE "\n"
 		       "dot11RSNAPairwiseCipherSelected=" RSN_SUITE "\n"
 		       "dot11RSNAGroupCipherSelected=" RSN_SUITE "\n"
@@ -2226,13 +2454,6 @@ int wpa_sm_get_mib(struct wpa_sm *sm, char *buf, size_t buflen)
 		       "dot11RSNAGroupCipherRequested=" RSN_SUITE "\n"
 		       "dot11RSNAConfigNumberOfGTKSAReplayCounters=0\n"
 		       "dot11RSNA4WayHandshakeFailures=%u\n",
-		       rsna ? "TRUE" : "FALSE",
-		       rsna ? "TRUE" : "FALSE",
-		       RSN_VERSION,
-		       wpa_cipher_bits(sm->group_cipher),
-		       sm->dot11RSNAConfigPMKLifetime,
-		       sm->dot11RSNAConfigPMKReauthThreshold,
-		       sm->dot11RSNAConfigSATimeout,
 		       RSN_SUITE_ARG(wpa_key_mgmt_suite(sm)),
 		       RSN_SUITE_ARG(wpa_cipher_suite(sm,
 						      sm->pairwise_cipher)),
@@ -2243,13 +2464,40 @@ int wpa_sm_get_mib(struct wpa_sm *sm, char *buf, size_t buflen)
 						      sm->pairwise_cipher)),
 		       RSN_SUITE_ARG(wpa_cipher_suite(sm, sm->group_cipher)),
 		       sm->dot11RSNA4WayHandshakeFailures);
-	return len;
+
+	return (int) len;
+}
+
+
+static void wpa_sm_pmksa_free_cb(struct rsn_pmksa_cache_entry *entry,
+				 void *ctx, int replace)
+{
+	struct wpa_sm *sm = ctx;
+
+	if (sm->cur_pmksa == entry ||
+	    (sm->pmk_len == entry->pmk_len &&
+	     memcmp(sm->pmk, entry->pmk, sm->pmk_len) == 0)) {
+		wpa_printf(MSG_DEBUG, "RSN: removed current PMKSA entry");
+		sm->cur_pmksa = NULL;
+
+		if (replace) {
+			/* A new entry is being added, so no need to
+			 * deauthenticate in this case. This happens when EAP
+			 * authentication is completed again (reauth or failed
+			 * PMKSA caching attempt). */
+			return;
+		}
+
+		memset(sm->pmk, 0, sizeof(sm->pmk));
+		wpa_sm_deauthenticate(sm, REASON_UNSPECIFIED);
+		wpa_sm_req_scan(sm, 0, 0);
+	}
 }
 
 
 /**
  * wpa_sm_init - Initialize WPA state machine
- * @ctx: Context pointer for callbacks; this needs to be an allocated buffer
+ * @ctx: Context pointer for callbacks
  * Returns: Pointer to the allocated WPA state machine data
  *
  * This function is used to allocate a new WPA state machine and the returned
@@ -2270,6 +2518,14 @@ struct wpa_sm * wpa_sm_init(struct wpa_sm_ctx *ctx)
 	sm->dot11RSNAConfigPMKReauthThreshold = 70;
 	sm->dot11RSNAConfigSATimeout = 60;
 
+	sm->pmksa = pmksa_cache_init(wpa_sm_pmksa_free_cb, sm, sm);
+	if (sm->pmksa == NULL) {
+		wpa_printf(MSG_ERROR, "RSN: PMKSA cache initialization "
+			   "failed");
+		free(sm);
+		return NULL;
+	}
+
 	return sm;
 }
 
@@ -2282,6 +2538,7 @@ void wpa_sm_deinit(struct wpa_sm *sm)
 {
 	if (sm == NULL)
 		return;
+	pmksa_cache_deinit(sm->pmksa);
 	eloop_cancel_timeout(wpa_sm_start_preauth, sm, 0);
 	free(sm->assoc_wpa_ie);
 	free(sm->ap_wpa_ie);
@@ -2407,8 +2664,10 @@ void wpa_sm_set_scard_ctx(struct wpa_sm *sm, void *scard_ctx)
  */
 void wpa_sm_set_config(struct wpa_sm *sm, struct wpa_ssid *config)
 {
-	if (sm)
+	if (sm) {
 		sm->cur_ssid = config;
+		pmksa_cache_notify_reconfig(sm->pmksa);
+	}
 }
 
 
