@@ -1,6 +1,6 @@
 /*
  * wpa_supplicant - Internal definitions
- * Copyright (c) 2003-2006, Jouni Malinen <jkmaline@cc.hut.fi>
+ * Copyright (c) 2003-2005, Jouni Malinen <jkmaline@cc.hut.fi>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 2 as
@@ -28,6 +28,14 @@ struct wpa_scan_result;
 struct wpa_sm;
 struct wpa_supplicant;
 
+/*
+ * Forward declarations of private structures used within the ctrl_iface
+ * backends. Other parts of wpa_supplicant do not have access to data stored in
+ * these structures.
+ */
+struct ctrl_iface_priv;
+struct ctrl_iface_global_priv;
+
 /**
  * struct wpa_interface - Parameters for wpa_supplicant_add_iface()
  */
@@ -143,7 +151,7 @@ struct wpa_params {
 struct wpa_global {
 	struct wpa_supplicant *ifaces;
 	struct wpa_params params;
-	int ctrl_sock;
+	struct ctrl_iface_global_priv *ctrl_iface;
 };
 
 /**
@@ -164,7 +172,7 @@ struct wpa_supplicant {
 	char *confname;
 	struct wpa_config *conf;
 	int countermeasures;
-	time_t last_michael_mic_error;
+	os_time_t last_michael_mic_error;
 	u8 bssid[ETH_ALEN];
 	int reassociate; /* reassociation requested */
 	int disconnected; /* all connections disabled; i.e., do no reassociate
@@ -195,9 +203,7 @@ struct wpa_supplicant {
 	struct wpa_sm *wpa;
 	struct eapol_sm *eapol;
 
-	int ctrl_sock; /* UNIX domain socket for control interface or -1 if
-			* not used */
-	struct wpa_ctrl_dst *ctrl_dst;
+	struct ctrl_iface_priv *ctrl_iface;
 
 	wpa_states wpa_state;
 	int new_connection;
@@ -230,6 +236,7 @@ int wpa_supplicant_driver_init(struct wpa_supplicant *wpa_s,
 struct wpa_blacklist * wpa_blacklist_get(struct wpa_supplicant *wpa_s,
 					 const u8 *bssid);
 int wpa_blacklist_add(struct wpa_supplicant *wpa_s, const u8 *bssid);
+int wpa_blacklist_del(struct wpa_supplicant *wpa_s, const u8 *bssid);
 void wpa_blacklist_clear(struct wpa_supplicant *wpa_s);
 int wpa_supplicant_set_suites(struct wpa_supplicant *wpa_s,
 			      struct wpa_scan_result *bss,
@@ -266,9 +273,6 @@ struct wpa_global * wpa_supplicant_init(struct wpa_params *params);
 int wpa_supplicant_run(struct wpa_global *global);
 void wpa_supplicant_deinit(struct wpa_global *global);
 
-int wpa_supplicant_scard_init(struct wpa_supplicant *wpa_s,
-			      struct wpa_ssid *ssid);
-
 
 /* driver_ops */
 static inline void * wpa_drv_init(struct wpa_supplicant *wpa_s,
